'''test_cr_bin_path.py: tests for the cr_bin_path module'''
#
# Copyright (C) 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from clickreviews.cr_bin_path import ClickReviewBinPath
import clickreviews.cr_tests as cr_tests


class TestClickReviewBinPath(cr_tests.TestClickReview):
    """Tests for the bin-path review tool."""
    def setUp(self):
        super().setUp()
        self.set_test_pkgfmt("snap", "15.04")

    def _set_binary(self, entries, name=None):
        d = dict()
        if name is None:
            d['name'] = self.default_appname
        else:
            d['name'] = name
        for (key, value) in entries:
            d[key] = value
        self.set_test_pkg_yaml("binaries", [d])

        if 'exec' in d:
            self.set_test_bin_path(d['name'], d['exec'])
        else:
            self.set_test_bin_path(d['name'], d['name'])

    def test_check_path(self):
        '''Test check_path()'''
        self.set_test_bin_path(self.default_appname, "bin/foo.exe")
        c = ClickReviewBinPath(self.test_name)
        c.check_path()
        r = c.click_report
        # We should end up with 1 info
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_path_nonexecutable(self):
        '''Test check_path() - nonexecutable'''
        self.set_test_bin_path(self.default_appname, "bin/foo.nonexec")
        c = ClickReviewBinPath(self.test_name)
        c.check_path()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_required(self):
        '''Test check_snappy_required()'''
        self._set_binary([("exec", "bin/foo")])
        c = ClickReviewBinPath(self.test_name)
        c.check_snappy_required()
        r = c.click_report
        # Only 'name' is required at this time so 0s for all
        expected_counts = {'info': 0, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_required_empty_value(self):
        '''Test check_snappy_required() - empty exec'''
        self._set_binary([("exec", "")])
        c = ClickReviewBinPath(self.test_name)
        c.check_snappy_required()
        r = c.click_report
        # Only 'name' is required at this time so 0s for all
        expected_counts = {'info': 0, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_required_bad_value(self):
        '''Test check_snappy_required() - bad exec'''
        self._set_binary([("exec", [])])
        c = ClickReviewBinPath(self.test_name)
        c.check_snappy_required()
        r = c.click_report
        # Only 'name' is required at this time so 0s for all
        expected_counts = {'info': 0, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_required_multiple(self):
        '''Test check_snappy_required() - multiple'''
        self._set_binary([("exec", "bin/foo"),
                          ("description", "foo desc")])
        c = ClickReviewBinPath(self.test_name)
        c.check_snappy_required()
        r = c.click_report
        # Only 'name' is required at this time so 0s for all
        expected_counts = {'info': 0, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_optional(self):
        '''Test check_snappy_optional()'''
        self._set_binary([("description", "some description")])
        c = ClickReviewBinPath(self.test_name)
        c.check_snappy_optional()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_optional_empty_value(self):
        '''Test check_snappy_optional() - empty description'''
        self._set_binary([("description", "")])
        c = ClickReviewBinPath(self.test_name)
        c.check_snappy_optional()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_optional_bad_value(self):
        '''Test check_snappy_optional() - bad description'''
        self._set_binary([("description", [])])
        c = ClickReviewBinPath(self.test_name)
        c.check_snappy_optional()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_unknown(self):
        '''Test check_snappy_unknown()'''
        self._set_binary([("nonexistent", "foo")])
        c = ClickReviewBinPath(self.test_name)
        c.check_snappy_unknown()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 1, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_unknown_multiple(self):
        '''Test check_snappy_unknown() - multiple'''
        self._set_binary([("exec", "bin/foo"),
                          ("nonexistent", "foo")])
        c = ClickReviewBinPath(self.test_name)
        c.check_snappy_unknown()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 1, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_binary_description(self):
        '''Test check_binary_description()'''
        self._set_binary([("description", "some description")])
        c = ClickReviewBinPath(self.test_name)
        c.check_binary_description()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_binary_description_unspecified(self):
        '''Test check_binary_description() - unspecified'''
        self._set_binary([("exec", "foo")])
        c = ClickReviewBinPath(self.test_name)
        c.check_binary_description()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_binary_description_empty(self):
        '''Test check_binary_description() - empty'''
        self._set_binary([("description", "")])
        c = ClickReviewBinPath(self.test_name)
        c.check_binary_description()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)



'''cr_content_hub.py: click content-hub checks'''
#
# Copyright (C) 2014-2015 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function

from clickreviews.cr_common import ClickReview, error, open_file_read
import json
import os


class ClickReviewContentHub(ClickReview):
    '''This class represents click lint reviews'''
    def __init__(self, fn, overrides=None):
        peer_hooks = dict()
        my_hook = 'content-hub'
        peer_hooks[my_hook] = dict()
        peer_hooks[my_hook]['allowed'] = ClickReview.app_allowed_peer_hooks
        peer_hooks[my_hook]['required'] = []

        ClickReview.__init__(self, fn, "content_hub", peer_hooks=peer_hooks,
                             overrides=overrides)
        if not self.is_click and not self.is_snap1:
            return

        self.valid_keys = ['destination', 'share', 'source']

        self.content_hub_files = dict()  # click-show-files and tests
        self.content_hub = dict()

        if self.manifest is None:
            return

        for app in self.manifest['hooks']:
            if 'content-hub' not in self.manifest['hooks'][app]:
                # msg("Skipped missing content-hub hook for '%s'" % app)
                continue
            if not isinstance(self.manifest['hooks'][app]['content-hub'], str):
                error("manifest malformed: hooks/%s/urls is not str" % app)
            (full_fn, jd) = self._extract_content_hub(app)
            self.content_hub_files[app] = full_fn
            self.content_hub[app] = jd

    def _extract_content_hub(self, app):
        '''Get content-hub hook content'''
        c = self.manifest['hooks'][app]['content-hub']
        fn = os.path.join(self.unpack_dir, c)

        bn = os.path.basename(fn)
        if not os.path.exists(fn):
            error("Could not find '%s'" % bn)

        fh = open_file_read(fn)
        contents = ""
        for line in fh.readlines():
            contents += line
        fh.close()

        try:
            jd = json.loads(contents)
        except Exception as e:
            error("content-hub json unparseable: %s (%s):\n%s" % (bn,
                  str(e), contents))

        if not isinstance(jd, dict):
            error("content-hub json is malformed: %s:\n%s" % (bn, contents))

        return (fn, jd)

    def check_valid(self):
        '''Check validity of content-hub entries'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.content_hub):
            for k in self.content_hub[app].keys():
                t = "info"
                n = self._get_check_name('valid', app=app, extra=k)
                s = "OK"

                if not isinstance(self.content_hub[app][k], list):
                    t = "error"
                    s = "'%s' is not a list" % k
                elif len(self.content_hub[app][k]) < 1:
                    t = "error"
                    s = "'%s' is empty" % k
                self._add_result(t, n, s)
                if t == "error":
                    continue

                for v in self.content_hub[app][k]:
                    t = "info"
                    n = self._get_check_name('valid_value', app=app, extra=k)
                    s = "OK"
                    if not isinstance(v, str):
                        t = "error"
                        s = "'%s' is not a string" % k
                    elif v == "":
                        t = "error"
                        s = "'%s' is empty" % k
                    self._add_result(t, n, s)

    def check_unknown_keys(self):
        '''Check unknown'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.content_hub):
            unknown = []
            t = "info"
            n = self._get_check_name('unknown', app=app)
            s = "OK"
            for key in self.content_hub[app].keys():
                if key not in self.valid_keys:
                    unknown.append(key)
            if len(unknown) == 1:
                t = "warn"
                s = "Unknown field '%s'" % unknown[0]
            elif len(unknown) > 1:
                t = "warn"
                s = "Unknown fields '%s'" % ", ".join(unknown)
            self._add_result(t, n, s)



'''test_cr_scope.py: tests for the cr_scope module'''
#
# Copyright (C) 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from clickreviews.cr_scope import ClickReviewScope
import clickreviews.cr_tests as cr_tests
import configparser


class TestClickReviewScope(cr_tests.TestClickReview):
    """Tests for the lint review tool."""

    def _create_scope(self, config_dict=None):
        '''Create a scope to pass to tests'''
        scope = dict()
        scope["dir_rel"] = "scope-directory"
        scope["ini_file_rel"] = "%s/%s_%s.ini" % (scope["dir_rel"],
                                                  self.default_appname,
                                                  'foo')
        scope["scope_config"] = configparser.ConfigParser()
        scope["scope_config"]['ScopeConfig'] = config_dict

        return scope

    def _stub_config(self):
        '''Stub configparser file'''
        config_dict = {
            'ScopeRunner': "%s" % self.default_appname,
            'ChildScopes': 'Child1',
            'DisplayName': 'Foo',
            'Description': 'Some description',
            'Author': 'Foo Ltd.',
            'Art': '',
            'Icon': 'foo.svg',
            'SearchHint': 'Search Foo',
            'HotKey': 'h',
            'IdleTimeout': '1234',
            'Invisible': 'false',
            'LocationDataNeeded': 'false',
            'ResultsTtlType': 'small',
        }

        return config_dict

    def test_check_scope_ini(self):
        '''Test check_scope_ini()'''
        scope = self._create_scope(self._stub_config())

        self.set_test_scope(self.default_appname, scope)
        c = ClickReviewScope(self.test_name)
        c.check_scope_ini()
        r = c.click_report
        expected_counts = {'info': 4, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_scope_ini_missing_required1(self):
        '''Test check_scope_ini() - missing Description'''
        config = self._stub_config()
        del config['Description']
        scope = self._create_scope(config)

        self.set_test_scope(self.default_appname, scope)
        c = ClickReviewScope(self.test_name)
        c.check_scope_ini()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_scope_ini_missing_required2(self):
        '''Test check_scope_ini() - missing DisplayName'''
        config = self._stub_config()
        del config['DisplayName']
        scope = self._create_scope(config)

        self.set_test_scope(self.default_appname, scope)
        c = ClickReviewScope(self.test_name)
        c.check_scope_ini()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_scope_ini_missing_required3(self):
        '''Test check_scope_ini() - missing Author'''
        config = self._stub_config()
        del config['Author']
        scope = self._create_scope(config)

        self.set_test_scope(self.default_appname, scope)
        c = ClickReviewScope(self.test_name)
        c.check_scope_ini()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_scope_ini_missing_required4(self):
        '''Test check_scope_ini() - missing multiple'''
        config = self._stub_config()
        del config['Description']
        del config['DisplayName']
        del config['Author']
        scope = self._create_scope(config)

        self.set_test_scope(self.default_appname, scope)
        c = ClickReviewScope(self.test_name)
        c.check_scope_ini()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_scope_ini_translated_field(self):
        '''Test check_scope_ini() - translated field - es'''
        config = self._stub_config()
        config['searchhint[es]'] = "foo"
        scope = self._create_scope(config)

        self.set_test_scope(self.default_appname, scope)
        c = ClickReviewScope(self.test_name)
        c.check_scope_ini()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_scope_ini_translated_field2(self):
        '''Test check_scope_ini() - translated field - pt_br'''
        config = self._stub_config()
        config['searchhint[pt_br]'] = "foo"
        scope = self._create_scope(config)

        self.set_test_scope(self.default_appname, scope)
        c = ClickReviewScope(self.test_name)
        c.check_scope_ini()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_scope_ini_translated_field3(self):
        '''Test check_scope_ini() - translated field - ast'''
        config = self._stub_config()
        config['searchhint[ast]'] = "foo"
        scope = self._create_scope(config)

        self.set_test_scope(self.default_appname, scope)
        c = ClickReviewScope(self.test_name)
        c.check_scope_ini()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_scope_ini_bad_translated_field(self):
        '''Test check_scope_ini() - bad translated field'''
        config = self._stub_config()
        config['searchhint[ba;r]'] = "foo"
        scope = self._create_scope(config)

        self.set_test_scope(self.default_appname, scope)
        c = ClickReviewScope(self.test_name)
        c.check_scope_ini()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 1, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_scope_ini_bad_translated_field2(self):
        '''Test check_scope_ini() - translated field - de_DE'''
        config = self._stub_config()
        config['searchhint[de_DE]'] = "foo"
        scope = self._create_scope(config)

        self.set_test_scope(self.default_appname, scope)
        c = ClickReviewScope(self.test_name)
        c.check_scope_ini()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_scope_ini_nonexistent_field(self):
        '''Test check_scope_ini() - non-existent field'''
        config = self._stub_config()
        config['nonexistent'] = "foo"
        scope = self._create_scope(config)

        self.set_test_scope(self.default_appname, scope)
        c = ClickReviewScope(self.test_name)
        c.check_scope_ini()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 1, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_scope_ini_forbidden_field(self):
        '''Test check_scope_ini() - forbidden field'''
        config = self._stub_config()
        config['debugmode'] = "true"
        scope = self._create_scope(config)

        self.set_test_scope(self.default_appname, scope)
        c = ClickReviewScope(self.test_name)
        c.check_scope_ini()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_peer_hooks(self):
        '''Test check_peer_hooks()'''
        c = ClickReviewScope(self.test_name)

        # create a new hooks database for our peer hooks tests
        tmp = dict()

        # add our hook
        tmp["scope"] = "scope.ini"

        # add any required peer hooks
        tmp["apparmor"] = "foo.apparmor"

        # update the manifest and test_manifest
        c.manifest["hooks"][self.default_appname] = tmp
        self._update_test_manifest()

        # do the test
        c.check_peer_hooks()
        r = c.click_report
        # We should end up with 2 info
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_peer_hooks_disallowed(self):
        '''Test check_peer_hooks() - disallowed'''
        c = ClickReviewScope(self.test_name)

        # create a new hooks database for our peer hooks tests
        tmp = dict()

        # add our hook
        tmp["scope"] = "scope.ini"

        # add any required peer hooks
        tmp["apparmor"] = "foo.apparmor"

        # add something not allowed
        tmp["desktop"] = "foo.desktop"

        # update the manifest and test_manifest
        c.manifest["hooks"][self.default_appname] = tmp
        self._update_test_manifest()

        # do the test
        c.check_peer_hooks()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_peer_hooks_required(self):
        '''Test check_peer_hooks() - required'''
        c = ClickReviewScope(self.test_name)

        # create a new hooks database for our peer hooks tests
        tmp = dict()

        # add our hook
        tmp["scope"] = "scope.ini"

        # skip adding required hooks

        # update the manifest and test_manifest
        c.manifest["hooks"][self.default_appname] = tmp
        self._update_test_manifest()

        # do the test
        c.check_peer_hooks()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_scope_ini_snappy_1504(self):
        '''Test check_scope_ini() - snappy 15.04'''
        self.set_test_pkgfmt("snap", "15.04")
        scope = self._create_scope(self._stub_config())

        self.set_test_scope(self.default_appname, scope)
        c = ClickReviewScope(self.test_name)
        c.check_scope_ini()
        r = c.click_report
        expected_counts = {'info': 4, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)



'''cr_skeleton.py: click skeleton'''
#
# Copyright (C) 2014-2015 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function

from clickreviews.cr_common import ClickReview


class ClickReviewSkeleton(ClickReview):
    '''This class represents click lint reviews'''
    def __init__(self, fn, overrides=None):
        # Many test classes are for verify click hooks. 'peer_hooks' is used
        # to declare what hooks may be use with my_hook. When using this
        # mechanism, ClickReview.check_peer_hooks() is run for you.
        peer_hooks = dict()
        my_hook = 'skeleton'
        peer_hooks[my_hook] = dict()
        peer_hooks[my_hook]['allowed'] = ["desktop", "apparmor", "urls"]
        peer_hooks[my_hook]['required'] = ["desktop", "apparmor"]

        ClickReview.__init__(self, fn, "skeleton", peer_hooks=peer_hooks,
                             overrides=overrides)

        if not self.is_click and not self.is_snap1:
            return

        # If not a hooks test, skip the above and omit peer_hooks like so:
        # ClickReview.__init__(self, fn, "skeleton")

    def check_foo(self):
        '''Check foo'''
        if not self.is_click and not self.is_snap1:
            return

        t = 'info'
        n = self._get_check_name('foo')
        s = "OK"
        if False:
            t = 'error'
            s = "some message"
        self._add_result(t, n, s)

    def check_bar(self):
        '''Check bar'''
        if not self.is_click and not self.is_snap1:
            return

        t = 'info'
        n = self._get_check_name('bar')
        s = "OK"
        if True:
            t = 'error'
            s = "some message"
        self._add_result(t, n, s)

    def check_baz(self):
        '''Check baz'''
        if not self.is_click and not self.is_snap1:
            return

        n = self._get_check_name('baz')
        self._add_result('warn', n, 'TODO', link="http://example.com")

        # Spawn a shell to pause the script (run 'exit' to continue)
        # import subprocess
        # print(self.unpack_dir)
        # subprocess.call(['bash'])



from clickreviews.cr_common import ClickReview
from clickreviews import cr_tests


class ClickReviewTestCase(cr_tests.TestClickReview):

    def setUp(self):
        super().setUp()
        self.review = ClickReview('app.click', 'review_type')

    def test_add_result_default_manual_review(self):
        self.review._add_result('info', 'some-check', 'OK')
        self.assertEqual(self.review.click_report, {
            'info': {
                'some-check': {
                    'text': 'OK',
                    'manual_review': False,
                }
            },
            'warn': {},
            'error': {},
        })

    def test_add_result_custom_manual_review(self):
        self.review._add_result('info', 'some-check', 'OK',
                                manual_review=True)
        self.assertEqual(self.review.click_report, {
            'info': {
                'some-check': {
                    'text': 'OK',
                    'manual_review': True,
                }
            },
            'warn': {},
            'error': {},
        })

    def test_add_result_override_result_type_warn(self):
        self.review._add_result('warn', 'some-check', 'notok',
                                override_result_type='info')
        self.assertEqual(self.review.click_report, {
            'info': {
                'some-check': {
                    'text': '[WARN] notok',
                    'manual_review': False,
                }
            },
            'warn': {},
            'error': {},
        })

    def test_add_result_override_result_type_error(self):
        self.review._add_result('error', 'some-check', 'notok',
                                override_result_type='info')
        self.assertEqual(self.review.click_report, {
            'info': {
                'some-check': {
                    'text': '[ERROR] notok',
                    'manual_review': False,
                }
            },
            'warn': {},
            'error': {},
        })

    def test_add_result_override_result_type_info(self):
        self.review._add_result('info', 'some-check', 'ok',
                                override_result_type='warn')
        self.assertEqual(self.review.click_report, {
            'warn': {
                'some-check': {
                    'text': '[INFO] ok',
                    'manual_review': False,
                }
            },
            'info': {},
            'error': {},
        })

    def test_verify_peer_hooks_empty(self):
        '''Check verify_peer_hooks() - empty'''
        peer_hooks = dict()
        my_hook = "foo"
        peer_hooks[my_hook] = dict()
        peer_hooks[my_hook]['allowed'] = []
        peer_hooks[my_hook]['required'] = []
        self.review.peer_hooks = peer_hooks

        d = self.review._verify_peer_hooks(my_hook)
        self.assertEqual(0, len(d.keys()))

    def test_verify_peer_hooks_missing(self):
        '''Check verify_peer_hooks() - missing required'''
        peer_hooks = dict()
        my_hook = "desktop"
        peer_hooks[my_hook] = dict()
        peer_hooks[my_hook]['allowed'] = ["apparmor", "urls"]
        peer_hooks[my_hook]['required'] = ["nonexistent"]
        self.review.peer_hooks = peer_hooks

        d = self.review._verify_peer_hooks(my_hook)
        self.assertEqual(1, len(d.keys()))
        self.assertTrue('missing' in d.keys())
        self.assertTrue('nonexistent' in d['missing'][self.default_appname])

    def test_verify_peer_hooks_disallowed(self):
        '''Check verify_peer_hooks() - disallowed'''
        peer_hooks = dict()
        my_hook = "desktop"
        peer_hooks[my_hook] = dict()
        peer_hooks[my_hook]['allowed'] = ["apparmor"]
        peer_hooks[my_hook]['required'] = []
        self.review.peer_hooks = peer_hooks

        d = self.review._verify_peer_hooks(my_hook)
        self.assertEqual(1, len(d.keys()))
        self.assertTrue('disallowed' in d.keys())
        self.assertTrue('urls' in d['disallowed'][self.default_appname])

    def test_get_check_name(self):
        name = self.review._get_check_name('prefix')
        self.assertEqual(name, 'review_type:prefix')

    def test_get_check_name_with_app(self):
        name = self.review._get_check_name('prefix', app='app')
        self.assertEqual(name, 'review_type:prefix:app')

    def test_get_check_name_with_extra(self):
        name = self.review._get_check_name('prefix', extra='extra')
        self.assertEqual(name, 'review_type:prefix:extra')

    def test_get_check_name_with_app_and_extra(self):
        name = self.review._get_check_name('prefix', app='app', extra='extra')
        self.assertEqual(name, 'review_type:prefix:app:extra')

    def test_check_if_message_catalog_true(self):
        self.assertTrue(self.review._check_if_message_catalog('/a/b/foo.mo'))

    def test_check_if_message_catalog_false(self):
        self.assertFalse(self.review._check_if_message_catalog('/a/b/foo.txt'))



'''cr_systemd.py: click systemd'''
#
# Copyright (C) 2015 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function

from clickreviews.cr_common import ClickReview, error
import copy
import re


class ClickReviewSystemd(ClickReview):
    '''This class represents click lint reviews'''
    def __init__(self, fn, overrides=None):
        # systemd isn't implemented as a hook any more so don't setup peerhooks
        ClickReview.__init__(self, fn, "snappy-systemd", overrides=overrides)

        self.systemd_files = dict()  # click-show-files and tests
        self.systemd = dict()

        if not self.is_snap1:
            return

        # snappy-systemd currently only allows specifying:
        # - start (required)
        # - description (required)
        # - stop
        # - poststop
        # - stop-timeout
        # - caps (checked in in cr_security.py)
        # - security-template (checked in in cr_security.py)
        # - security-override (checked in in cr_security.py)
        # - security-policy (checked in in cr_security.py)
        self.required_keys = ['start', 'description']
        self.optional_keys = ['stop',
                              'poststop',
                              'stop-timeout',
                              'bus-name',
                              'listen-stream',
                              'socket',
                              'socket-user',
                              'socket-group',
                              'ports'
                              ] + self.snappy_exe_security

        if self.is_snap1 and 'services' in self.pkg_yaml:
            if len(self.pkg_yaml['services']) == 0:
                error("package.yaml malformed: 'services' is empty")
            for service in self.pkg_yaml['services']:
                if 'name' not in service:
                    error("package.yaml malformed: required 'name' not found "
                          "for entry in %s" % self.pkg_yaml['services'])
                elif not isinstance(service['name'], str):
                    error("package.yaml malformed: required 'name' is not str"
                          "for entry in %s" % self.pkg_yaml['services'])

                app = service['name']
                self.systemd[app] = copy.deepcopy(service)
                del self.systemd[app]['name']

    def _verify_required(self, my_dict, test_str):
        for app in sorted(my_dict):
            for r in self.required_keys:
                found = False
                t = 'info'
                n = self._get_check_name(
                    '%s_required_key' % test_str, extra=r, app=app)
                s = "OK"
                if r in my_dict[app]:
                    if not isinstance(my_dict[app][r], str):
                        t = 'error'
                        s = "'%s' is not a string" % r
                    elif my_dict[app][r] == "":
                        t = 'error'
                        s = "'%s' is empty" % r
                    else:
                        found = True
                if not found and t != 'error':
                    t = 'error'
                    s = "Missing required field '%s'" % r
                self._add_result(t, n, s)

    def check_snappy_required(self):
        '''Check for package.yaml required fields'''
        if not self.is_snap1 or 'services' not in self.pkg_yaml:
            return
        self._verify_required(self._create_dict(self.pkg_yaml['services']),
                              'package_yaml')

    def _verify_optional(self, my_dict, test_str):
        for app in sorted(my_dict):
            for o in self.optional_keys:
                if o in self.snappy_exe_security:
                    continue  # checked in cr_security.py
                found = False
                t = 'info'
                n = self._get_check_name(
                    '%s_optional_key' % test_str, extra=o, app=app)
                s = "OK"
                if o in my_dict[app]:
                    if o == 'stop-timeout':
                        if isinstance(my_dict[app][o], int):
                            found = True
                        elif not isinstance(my_dict[app][o], str):
                            t = 'error'
                            s = "'%s' is not a string or integer" % o
                        elif not re.search(r'[0-9]+[ms]?$', my_dict[app][o]):
                            t = 'error'
                            s = "'%s' is not of form NN[ms] (%s)" % \
                                (my_dict[app][o], o)
                        else:
                            found = True
                    elif o == 'ports':
                        if not isinstance(my_dict[app][o], dict):
                            t = 'error'
                            s = "'%s' is not dictionary" % o
                    elif o == 'socket':
                        if not isinstance(my_dict[app][o], bool):
                            t = 'error'
                            s = "'%s' is not boolean" % o
                    elif not isinstance(my_dict[app][o], str):
                        t = 'error'
                        s = "'%s' is not a string" % o
                    elif my_dict[app][o] == "":
                        t = 'error'
                        s = "'%s' is empty" % o
                    else:
                        found = True
                if not found and t != 'error':
                    s = "OK (skip missing)"
                self._add_result(t, n, s)

    def check_snappy_optional(self):
        '''Check snappy packate.yaml optional fields'''
        if not self.is_snap1 or 'services' not in self.pkg_yaml:
            return
        self._verify_optional(self._create_dict(self.pkg_yaml['services']),
                              'package_yaml')

    def _verify_unknown(self, my_dict, test_str):
        for app in sorted(my_dict):
            unknown = []
            t = 'info'
            n = self._get_check_name('%s_unknown_key' % test_str, app=app)
            s = "OK"

            for f in my_dict[app].keys():
                if f not in self.required_keys and \
                   f not in self.optional_keys:
                    unknown.append(f)

            if len(unknown) == 1:
                t = 'warn'
                s = "Unknown field '%s'" % unknown[0]
            elif len(unknown) > 1:
                t = 'warn'
                s = "Unknown fields '%s'" % ", ".join(unknown)
            self._add_result(t, n, s)

    def check_snappy_unknown(self):
        '''Check snappy package.yaml unknown fields'''
        if not self.is_snap1 or 'services' not in self.pkg_yaml:
            return
        self._verify_unknown(self._create_dict(self.pkg_yaml['services']),
                             'package_yaml')

    def _verify_service_description(self, my_dict, test_str):
        '''Check snappy systemd description'''
        for app in sorted(my_dict):
            t = 'info'
            n = self._get_check_name('%s_description_present' % test_str,
                                     app=app)
            s = 'OK'
            if 'description' not in my_dict[app]:
                s = 'required description field not specified'
                self._add_result('error', n, s)
                return
            self._add_result(t, n, s)

            t = 'info'
            n = self._get_check_name('%s_description_empty' % test_str,
                                     app=app)
            s = 'OK'
            if len(my_dict[app]['description']) == 0:
                t = 'error'
                s = "description is empty"
            self._add_result(t, n, s)

    def check_snappy_service_description(self):
        '''Check snappy package.yaml description'''
        if not self.is_snap1 or 'services' not in self.pkg_yaml:
            return
        self._verify_service_description(self._create_dict(
                                         self.pkg_yaml['services']),
                                         'package_yaml')

    def _verify_entry(self, my_dict, d, test_str):
        for app in sorted(my_dict):
            if d not in my_dict[app]:
                continue

            t = 'info'
            n = self._get_check_name('%s_empty' % test_str, extra=d, app=app)
            s = 'OK'
            if len(my_dict[app][d]) == 0:
                t = 'error'
                s = "%s entry is empty" % d
                self._add_result(t, n, s)
                continue
            self._add_result(t, n, s)

            t = 'info'
            n = self._get_check_name('%s_absolute_path' % test_str,
                                     extra=d, app=app)
            s = 'OK'
            if my_dict[app][d].startswith('/'):
                t = 'error'
                s = "'%s' should not specify absolute path" % my_dict[app][d]
            self._add_result(t, n, s)

    def check_snappy_service_start(self):
        '''Check snappy package.yaml start'''
        if not self.is_snap1 or 'services' not in self.pkg_yaml:
            return
        self._verify_entry(self._create_dict(self.pkg_yaml['services']),
                           'start', 'package_yaml')

    def check_snappy_service_stop(self):
        '''Check snappy package.yaml stop'''
        if not self.is_snap1 or 'services' not in self.pkg_yaml:
            return
        self._verify_entry(self._create_dict(self.pkg_yaml['services']),
                           'stop', 'package_yaml')

    def check_snappy_service_poststop(self):
        '''Check snappy package.yaml poststop'''
        if not self.is_snap1 or 'services' not in self.pkg_yaml:
            return
        self._verify_entry(self._create_dict(self.pkg_yaml['services']),
                           'poststop', 'package_yaml')

    def _verify_service_stop_timeout(self, my_dict, test_str):
        for app in sorted(my_dict):
            t = 'info'
            n = self._get_check_name('%s_stop_timeout' % test_str, app=app)
            s = "OK"

            if 'stop-timeout' not in my_dict[app]:
                s = "OK (skip missing)"
                self._add_result(t, n, s)
                return

            st = my_dict[app]['stop-timeout']

            if not isinstance(st, int) and not isinstance(st, str):
                t = 'error'
                s = 'stop-timeout is not a string or integer'
                self._add_result(t, n, s)
                return

            if isinstance(st, str):
                if re.search(r'[0-9]+[ms]?$', st):
                    st = int(st.rstrip(r'[ms]'))
                else:
                    t = 'error'
                    s = "'%s' is not of form NN[ms] (%s)" % (my_dict[app], st)
                self._add_result(t, n, s)
                return

            if st < 0 or st > 60:
                t = 'error'
                s = "stop-timeout '%d' out of range (0-60)" % \
                    my_dict[app]['stop-timeout']

            self._add_result(t, n, s)

    def check_snappy_service_stop_timeout(self):
        '''Check snappy package.yaml stop-timeout'''
        if not self.is_snap1 or 'services' not in self.pkg_yaml:
            return
        self._verify_service_stop_timeout(self._create_dict(
                                          self.pkg_yaml['services']),
                                          'package_yaml')

    def _verify_service_bus_name(self, pkgname, my_dict, test_str, is_fwk):
        for app in sorted(my_dict):
            if 'bus-name' not in my_dict[app]:
                continue

            t = 'info'
            n = self._get_check_name('%s_bus-name_framework' % test_str,
                                     app=app)
            s = 'OK'
            if not is_fwk:
                t = 'error'
                s = "Use of bus-name requires package be of 'type: framework'"
            self._add_result(t, n, s)

            t = 'info'
            n = self._get_check_name('%s_bus-name_empty' % test_str, app=app)
            s = 'OK'
            if len(my_dict[app]['bus-name']) == 0:
                t = 'error'
                s = "'bus-name' is empty"
                self._add_result(t, n, s)
                continue
            self._add_result(t, n, s)

            t = 'info'
            n = self._get_check_name('%s_bus-name_format' % test_str, app=app)
            link = None
            s = 'OK'
            if not re.search(r'^[A-Za-z0-9][A-Za-z0-9_-]*'
                             r'(\.[A-Za-z0-9][A-Za-z0-9_-]*)+$',
                             my_dict[app]['bus-name']):
                t = 'error'
                link = ('http://dbus.freedesktop.org/doc/'
                        'dbus-specification.html')
                s = ("'%s' is not of form '^[A-Za-z0-9][A-Za-z0-9_-]*"
                     "(\\.[A-Za-z0-9][A-Za-z0-9_-]*)+$'" %
                     (my_dict[app]['bus-name']))
            self._add_result(t, n, s, link)

            t = 'info'
            n = self._get_check_name('%s_bus-name_matches_name' % test_str,
                                     app=app)
            s = 'OK'
            suggested = [pkgname,
                         "%s.%s" % (pkgname, app)
                         ]
            found = False
            for name in suggested:
                if my_dict[app]['bus-name'].endswith(name):
                    found = True
                    break
            if not found:
                t = 'error'
                s = "'%s' doesn't end with one of: %s" % \
                    (my_dict[app]['bus-name'], ", ".join(suggested))
            self._add_result(t, n, s)

    def check_snappy_service_bus_name(self):
        '''Check snappy package.yaml bus-name'''
        if not self.is_snap1 or 'services' not in self.pkg_yaml:
            return
        is_framework = False
        if 'type' in self.pkg_yaml and self.pkg_yaml['type'] == 'framework':
            is_framework = True

        self._verify_service_bus_name(self.pkg_yaml['name'],
                                      self._create_dict(
                                          self.pkg_yaml['services']),
                                      'package_yaml',
                                      is_framework)

    def _verify_service_ports(self, pkgname, my_dict, test_str):
        for app in sorted(my_dict):
            if 'ports' not in my_dict[app]:
                continue

            t = 'info'
            n = self._get_check_name('%s_ports_empty' % test_str, app=app)
            s = 'OK'
            if len(my_dict[app]['ports'].keys()) == 0:
                t = 'error'
                s = "'ports' must contain 'internal' and/or 'external'"
                self._add_result(t, n, s)
                continue
            self._add_result(t, n, s)

            t = 'info'
            n = self._get_check_name('%s_ports_bad_key' % test_str, app=app)
            s = 'OK'
            badkeys = []
            for i in my_dict[app]['ports'].keys():
                if i not in ['internal', 'external']:
                    badkeys.append(i)
            if len(badkeys) > 0:
                t = 'error'
                s = "Unknown '%s' found in 'ports'" % ",".join(badkeys)
            self._add_result(t, n, s)

            port_pat = re.compile(r'^[0-9]+/[a-z0-9\-]+$')
            for key in ['internal', 'external']:
                if key not in my_dict[app]['ports']:
                    continue

                if len(my_dict[app]['ports'][key].keys()) < 1:
                    t = 'error'
                    n = self._get_check_name('%s_ports' % test_str, extra=key,
                                             app=app)
                    s = 'Could not find any %s ports' % key
                    self._add_result(t, n, s)
                    continue

                for tagname in my_dict[app]['ports'][key]:
                    entry = my_dict[app]['ports'][key][tagname]
                    if len(entry.keys()) < 1:
                        t = 'error'
                        n = self._get_check_name('%s_ports' % test_str,
                                                 extra=key, app=app)
                        s = 'Could not find any subkeys for %s' % tagname
                        self._add_result(t, n, s)
                        continue
                    # Annoyingly, the snappy-systemd file uses 'Port' and
                    # 'Negotiable' instead of 'port' and 'negotiable' from the
                    # yaml
                    if (test_str == 'package_yaml' and
                            'negotiable' not in entry and
                            'port' not in entry) or \
                       (test_str == 'hook' and
                            'Negotiable' not in entry and
                            'Port' not in entry):
                        t = 'error'
                        n = self._get_check_name('%s_ports_invalid' % test_str,
                                                 extra=key, app=app)
                        s = "Must specify specify at least 'port' or " + \
                            "'negotiable'"
                        self._add_result(t, n, s)
                        continue

                    # port
                    subkey = 'port'
                    if test_str == 'hook':
                        subkey = 'Port'
                    t = 'info'
                    n = self._get_check_name('%s_ports_%s_format' %
                                             (test_str, tagname),
                                             extra=subkey)
                    s = 'OK'
                    if subkey not in entry:
                        s = 'OK (skipped, not found)'
                    else:
                        tmp = entry[subkey].split('/')
                        if not port_pat.search(entry[subkey]) or \
                           int(tmp[0]) < 1 or int(tmp[0]) > 65535:
                            t = 'error'
                            s = "'%s' should be of form " % entry[subkey] + \
                                "'port/protocol' where port is an integer " + \
                                "(1-65535) and protocol is found in " + \
                                "/etc/protocols"
                    self._add_result(t, n, s)

                    # negotiable
                    subkey = 'negotiable'
                    if test_str == 'hook':
                        subkey = 'Negotiable'
                    t = 'info'
                    n = self._get_check_name('%s_ports_%s_format' %
                                             (test_str, tagname),
                                             extra=subkey)
                    s = 'OK'
                    if subkey not in entry:
                        s = 'OK (skipped, not found)'
                    elif entry[subkey] not in [True, False]:
                        t = 'error'
                        s = "'%s: %s' should be either 'yes' or 'no'" % \
                            (subkey, entry[subkey])
                    self._add_result(t, n, s)

    def check_snappy_service_ports(self):
        '''Check snappy package.yaml ports'''
        if not self.is_snap1 or 'services' not in self.pkg_yaml:
            return
        self._verify_service_ports(self.pkg_yaml['name'],
                                   self._create_dict(
                                       self.pkg_yaml['services']),
                                   'package_yaml')

    def _verify_service_listen_stream(self, pkgname, my_dict, test_str):
        for app in sorted(my_dict):
            if 'listen-stream' not in my_dict[app]:
                continue

            t = 'info'
            n = self._get_check_name('%s_listen-stream_empty' % test_str,
                                     app=app)
            s = 'OK'
            if len(my_dict[app]['listen-stream']) == 0:
                t = 'error'
                s = "'listen-stream' is empty"
                self._add_result(t, n, s)
                continue
            self._add_result(t, n, s)

            t = 'info'
            n = self._get_check_name('%s_listen-stream_matches_name' %
                                     test_str, app=app)
            s = 'OK'
            sock = my_dict[app]['listen-stream']
            if sock.startswith('@'):
                if sock != '@%s' % pkgname and \
                        not sock.startswith('@%s_' % pkgname):
                    t = 'error'
                    s = ("abstract socket '%s' is neither '%s' nor starts "
                         "with '%s'" % (sock, '@%s' % pkgname,
                                        '@%s_' % pkgname))
            elif sock.startswith('/'):
                found = False
                for path in ["/tmp/",
                             "/var/lib/apps/%s/" % pkgname,
                             "/var/lib/apps/%s." % pkgname,
                             "/run/shm/snaps/%s/" % pkgname,
                             "/run/shm/snaps/%s." % pkgname]:
                    if sock.startswith(path):
                        found = True
                        break
                if not found:
                    t = 'error'
                    s = ("named socket '%s' should be in a writable"
                         "app-specific area or /tmp" % sock)
            else:
                t = 'error'
                s = ("'%s' does not specify an abstract socket (starts "
                     "with '@') or absolute filename" % (sock))
            self._add_result(t, n, s)

    def check_snappy_service_listen_stream(self):
        '''Check snappy package.yaml listen-stream'''
        if not self.is_snap1 or 'services' not in self.pkg_yaml:
            return

        self._verify_service_listen_stream(self.pkg_yaml['name'],
                                           self._create_dict(
                                               self.pkg_yaml['services']),
                                           'package_yaml')

    def check_snappy_service_socket_user(self):
        '''Check snappy package.yaml socket-user'''
        if not self.is_snap1 or 'services' not in self.pkg_yaml:
            return

        my_dict = self._create_dict(self.pkg_yaml['services'])
        for app in sorted(my_dict):
            if 'socket-user' not in my_dict[app]:
                continue

            t = 'error'
            n = self._get_check_name('package_yaml_socket-user', app=app)
            s = "'socket-user' should not be used until snappy supports " + \
                "per-app users"
            self._add_result(t, n, s, manual_review=True)

            t = 'info'
            n = self._get_check_name('package_yaml_socket-user_matches',
                                     app=app)
            s = "OK"
            if my_dict[app]['socket-user'] != self.pkg_yaml['name']:
                t = 'error'
                s = "'%s' != '%s'" % (my_dict[app]['socket-user'],
                                      self.pkg_yaml['name'])
            self._add_result(t, n, s)

            t = 'info'
            n = self._get_check_name('package_yaml_socket-user_listen-stream',
                                     app=app)
            s = "OK"
            if 'listen-stream' not in my_dict[app]:
                t = 'error'
                s = "'socket-user' specified without 'listen-stream'"
            self._add_result(t, n, s)

    def check_snappy_service_socket_group(self):
        '''Check snappy package.yaml socket-group'''
        if not self.is_snap1 or 'services' not in self.pkg_yaml:
            return

        my_dict = self._create_dict(self.pkg_yaml['services'])
        for app in sorted(my_dict):
            if 'socket-group' not in my_dict[app]:
                continue

            t = 'error'
            n = self._get_check_name('package_yaml_socket-group', app=app)
            s = "'socket-group' should not be used until snappy supports " + \
                "per-app groups"
            self._add_result(t, n, s, manual_review=True)

            t = 'info'
            n = self._get_check_name('package_yaml_socket-group_matches',
                                     app=app)
            s = "OK"
            if my_dict[app]['socket-group'] != self.pkg_yaml['name']:
                t = 'error'
                s = "'%s' != '%s'" % (my_dict[app]['socket-group'],
                                      self.pkg_yaml['name'])
            self._add_result(t, n, s)

            t = 'info'
            n = self._get_check_name('package_yaml_socket-group_listen-stream',
                                     app=app)
            s = "OK"
            if 'listen-stream' not in my_dict[app]:
                t = 'error'
                s = "'socket-group' specified without 'listen-stream'"
            self._add_result(t, n, s)

    def check_snappy_service_socket(self):
        '''Check snappy package.yaml socket'''
        if not self.is_snap1 or 'services' not in self.pkg_yaml:
            return

        my_dict = self._create_dict(self.pkg_yaml['services'])
        for app in sorted(my_dict):
            if 'socket' not in my_dict[app]:
                continue

            t = 'info'
            n = self._get_check_name('package_yaml_socket_listen-stream',
                                     app=app)
            s = "OK"
            if 'listen-stream' not in my_dict[app]:
                t = 'error'
                s = "'socket' specified without 'listen-stream'"
            self._add_result(t, n, s)



'''cr_framework.py: click framework'''
#
# Copyright (C) 2014-2015 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function

from clickreviews.cr_common import ClickReview, error, open_file_read
import glob
import os
import re


class ClickReviewFramework(ClickReview):
    '''This class represents click framework reviews'''
    def __init__(self, fn, overrides=None):
        ClickReview.__init__(self, fn, "framework", overrides=overrides)

        self.frameworks_file = dict()
        self.frameworks = dict()

        if not self.is_snap1:
            return

        if self.manifest is not None:
            for app in self.manifest['hooks']:
                if 'framework' not in self.manifest['hooks'][app]:
                    # msg("Skipped missing framework hook for '%s'" % app)
                    continue
                if not isinstance(self.manifest['hooks'][app]['framework'],
                                  str):
                    error("manifest malformed: hooks/%s/framework is not str" %
                          app)
                (full_fn, data) = self._extract_framework(app)
                self.frameworks_file[app] = full_fn
                self.frameworks[app] = data

        self.framework_policy_dirs = ['apparmor', 'seccomp']
        self.framework_policy_subdirs = ['templates', 'policygroups']
        (self.framework_policy, self.framework_policy_unknown) = \
            self._extract_framework_policy()

    def _extract_framework(self, app):
        '''Get framework for app'''
        rel = self.manifest['hooks'][app]['framework']
        fn = os.path.join(self.unpack_dir, rel)
        if not os.path.exists(fn):
            error("Could not find '%s'" % rel)

        data = dict()
        fh = open_file_read(fn)
        for line in fh.readlines():
            tmp = line.split(':')
            if len(tmp) != 2:
                continue
            data[tmp[0].strip()] = tmp[1].strip()
        fh.close()

        return (fn, data)

    def _extract_framework_policy(self):
        '''Get framework policy files'''
        policy_dict = dict()
        unknown = []
        fpdir = os.path.join(self.unpack_dir, "meta", "framework-policy")
        for i in glob.glob("%s/*" % fpdir):
            rel_i = os.path.basename(i)
            if not os.path.isdir(i) or rel_i not in self.framework_policy_dirs:
                unknown.append(os.path.relpath(i, self.unpack_dir))
                continue

            policy_dict[rel_i] = dict()
            for j in glob.glob("%s/*" % i):
                rel_j = os.path.basename(j)
                if not os.path.isdir(j) or \
                   rel_j not in self.framework_policy_subdirs:
                    unknown.append(os.path.relpath(j, self.unpack_dir))
                    continue

                policy_dict[rel_i][rel_j] = dict()
                for k in glob.glob("%s/*" % j):
                    rel_k = os.path.basename(k)
                    if not os.path.isfile(k):
                        unknown.append(os.path.relpath(k, self.unpack_dir))
                        continue

                    fh = open_file_read(k)
                    policy_dict[rel_i][rel_j][rel_k] = fh.read()
                    fh.close()
        return (policy_dict, unknown)

    def _has_framework_in_metadir(self):
        '''Check if snap has meta/<name>.framework'''
        if not self.is_snap1:
            return False

        return os.path.exists(os.path.join(self.unpack_dir, 'meta',
                                           '%s.framework' %
                                           self.pkg_yaml['name']))

    def check_framework_hook_obsolete(self):
        '''Check manifest doesn't specify 'framework' hook'''
        if not self.is_snap1:
            return

        t = 'info'
        n = self._get_check_name("obsolete_declaration")
        s = "OK"
        if len(self.frameworks) > 0:
            t = 'error'
            s = "framework hook found for '%s'" % \
                ",".join(sorted(self.frameworks))
        self._add_result(t, n, s)

    def check_snappy_framework_file_obsolete(self):
        '''Check snap doesn't ship .framework file'''
        if not self.is_snap1 or self.pkg_yaml['type'] != 'framework':
            return
        t = 'info'
        n = self._get_check_name("obsolete_framework_file")
        s = "OK"
        if self._has_framework_in_metadir():
            t = 'warn'
            s = "found '%s.framework' (safe to remove)" % self.pkg_yaml['name']
        self._add_result(t, n, s)

    def check_snappy_framework_depends(self):
        '''Check framework doesn't depend on other frameworks'''
        if not self.is_snap1 or self.pkg_yaml['type'] != 'framework':
            return
        t = 'info'
        n = self._get_check_name("dependency")
        s = "OK"
        if "frameworks" in self.pkg_yaml:
            t = 'error'
            s = "'type: framework' may not specify 'frameworks'"
        self._add_result(t, n, s)

    def check_snappy_framework_policy(self):
        '''Check framework ships at least some policy'''
        if not self.is_snap1 or self.pkg_yaml['type'] != 'framework':
            return

        t = 'info'
        n = self._get_check_name("policies")
        s = "OK"
        found = False
        for i in self.framework_policy_dirs:
            if i not in self.framework_policy:
                continue
            for j in self.framework_policy_subdirs:
                if j not in self.framework_policy[i]:
                    continue
                if len(self.framework_policy[i][j].keys()) > 0:
                    found = True
        if not found:
            t = 'warn'
            s = "security policy not found"
        self._add_result(t, n, s)

        t = 'info'
        n = self._get_check_name("policy_unknown")
        s = "OK"
        if len(self.framework_policy_unknown) > 0:
            t = 'warn'
            s = "framework policy has unexpected entries: '%s'" % \
                ",".join(self.framework_policy_unknown)
        self._add_result(t, n, s)

    def check_snappy_framework_policy_metadata(self):
        '''Check framework policy has expected meta data'''
        if not self.is_snap1 or self.pkg_yaml['type'] != 'framework':
            return

        t = 'info'
        n = self._get_check_name("policy_metadata")
        s = "OK"
        msgs = []
        for term in ["# Description: ", "# Usage: "]:
            for i in self.framework_policy_dirs:
                if i not in self.framework_policy:
                    continue
                for j in self.framework_policy_subdirs:
                    if j not in self.framework_policy[i]:
                        continue
                    for k in self.framework_policy[i][j].keys():
                        found = False
                        for l in self.framework_policy[i][j][k].splitlines():  # noqa
                            if l.startswith(term):
                                found = True
                        if not found:
                            msgs.append("'%s' in '%s/%s/%s'" % (term, i, j, k))
        if len(msgs) > 0:
            t = 'error'
            s = "Could not find meta data: %s" % ",".join(msgs)
        self._add_result(t, n, s)

    def check_snappy_framework_policy_matching(self):
        '''Check framework policy ships apparmor and seccomp for each'''
        if not self.is_snap1 or self.pkg_yaml['type'] != 'framework':
            return

        t = 'info'
        n = self._get_check_name("has_all_policy")
        s = "OK"
        if len(self.framework_policy.keys()) == 0:
            s = "OK (skipped missing policy)"
            self._add_result(t, n, s)
            return

        for i in self.framework_policy:
            for j in self.framework_policy[i]:
                for k in self.framework_policy[i][j]:
                    for other in self.framework_policy_dirs:
                        if t == i:
                            continue
                        t = 'info'
                        n = self._get_check_name(
                            "policy", extra="%s/%s/%s" % (i, j, k))
                        s = "OK"
                        if j not in self.framework_policy[other] or \
                           k not in self.framework_policy[other][j]:
                            t = 'error'
                            s = "Could not find matching '%s/%s/%s'" % (other,
                                                                        j, k)
                        self._add_result(t, n, s)

    def check_snappy_framework_policy_filenames(self):
        '''Check framework policy file names'''
        if not self.is_snap1 or self.pkg_yaml['type'] != 'framework':
            return

        for i in self.framework_policy:
            for j in self.framework_policy[i]:
                for k in self.framework_policy[i][j]:
                    f = "%s/%s/%s" % (i, j, k)
                    t = 'info'
                    n = self._get_check_name(
                        "policy_valid_name", extra=f)
                    s = "OK"
                    if not re.search(r'^[a-z0-9][a-z0-9+\.-]+$', k):
                        t = 'error'
                        s = r"'%s' should match '^[a-z0-9][a-z0-9+\.-]+$'" % f
                    elif k.startswith(self.pkg_yaml['name']):
                        t = 'warn'
                        s = "'%s' should not begin with package name" % f
                    self._add_result(t, n, s)



'''test_cr_systemd.py: tests for the cr_systemd module'''
#
# Copyright (C) 2015 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from clickreviews.cr_systemd import ClickReviewSystemd
import clickreviews.cr_tests as cr_tests


class TestClickReviewSystemd(cr_tests.TestClickReview):
    """Tests for the lint review tool."""
    def setUp(self):
        super().setUp()
        self.set_test_pkgfmt("snap", "15.04")

    def _create_ports(self):
        ports = {
            'internal': {'int1': {"port": '8081/tcp', "negotiable": True}},
            'external': {
                'ext1': {"port": '80/tcp', "negotiable": False},
                'ext2': {"port": '88/udp'}
            }
        }
        return ports

    def _set_service(self, entries, name=None):
        d = dict()
        if name is None:
            d['name'] = self.default_appname
        else:
            d['name'] = name
        for (key, value) in entries:
            d[key] = value
        self.set_test_pkg_yaml("services", [d])
        self.set_test_systemd(d['name'], 'name', d['name'])

    def test_check_snappy_required(self):
        '''Test check_required() - has start and description'''
        self.set_test_systemd(self.default_appname,
                              key="start",
                              value="bin/foo")
        self.set_test_systemd(self.default_appname,
                              key="description",
                              value="something")
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_required()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_required_empty_value(self):
        '''Test check_snappy_required() - empty start'''
        self.set_test_systemd(self.default_appname,
                              key="start",
                              value="")
        self.set_test_systemd(self.default_appname,
                              key="description",
                              value="something")
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_required()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_required_bad_value(self):
        '''Test check_snappy_required() - bad start'''
        self.set_test_systemd(self.default_appname,
                              key="start",
                              value=[])
        self.set_test_systemd(self.default_appname,
                              key="description",
                              value="something")
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_required()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_required_multiple(self):
        '''Test check_snappy_required() - multiple'''
        self.set_test_systemd(self.default_appname,
                              key="start",
                              value="/bin/foo")
        self.set_test_systemd(self.default_appname,
                              key="description",
                              value="something")
        self.set_test_systemd(self.default_appname,
                              key="stop",
                              value="/bin/foo-stop")
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_required()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_required_multiple2(self):
        '''Test check_snappy_required() - multiple with nonexistent'''
        self.set_test_systemd(self.default_appname,
                              key="start",
                              value="/bin/foo")
        self.set_test_systemd(self.default_appname,
                              key="description",
                              value="something")
        self.set_test_systemd(self.default_appname,
                              key="nonexistent",
                              value="foo")
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_required()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_optional_none(self):
        '''Test check_snappy_optional() - start only'''
        self.set_test_systemd(self.default_appname,
                              key="start",
                              value="/bin/foo")
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_optional()
        r = c.click_report
        expected_counts = {'info': 9, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_optional_stop_empty(self):
        '''Test check_snappy_optional() - with empty stop'''
        self.set_test_systemd(self.default_appname,
                              key="start",
                              value="/bin/foo")
        self.set_test_systemd(self.default_appname,
                              key="stop",
                              value="")
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_optional()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_optional_stop_bad(self):
        '''Test check_snappy_optional() - with bad stop'''
        self.set_test_systemd(self.default_appname,
                              key="start",
                              value="/bin/foo")
        self.set_test_systemd(self.default_appname,
                              key="stop",
                              value=[])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_optional()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_optional_stop_nonexistent(self):
        '''Test check_snappy_optional() - with stop plus nonexistent'''
        self.set_test_systemd(self.default_appname,
                              key="start",
                              value="/bin/foo")
        self.set_test_systemd(self.default_appname,
                              key="stop",
                              value="bin/bar")
        self.set_test_systemd(self.default_appname,
                              key="nonexistent",
                              value="foo")
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_optional()
        r = c.click_report
        expected_counts = {'info': 9, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_optional_stop_without_start(self):
        '''Test check_snappy_optional() - with stop, no start'''
        self.set_test_systemd(self.default_appname,
                              key="stop",
                              value="/bin/bar")
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_optional()
        r = c.click_report
        expected_counts = {'info': 9, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_optional_stop_without_start2(self):
        '''Test check_snappy_optional() - with stop, nonexistent, no start'''
        self.set_test_systemd(self.default_appname,
                              key="stop",
                              value="/bin/bar")
        self.set_test_systemd(self.default_appname,
                              key="nonexistent",
                              value="example.com")
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_optional()
        r = c.click_report
        expected_counts = {'info': 9, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_unknown(self):
        '''Test check_snappy_unknown()'''
        self.set_test_systemd(self.default_appname,
                              key="nonexistent",
                              value="foo")
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_unknown()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 1, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_unknown_multiple(self):
        '''Test check_snappy_unknown() - multiple with nonexistent'''
        self.set_test_systemd(self.default_appname,
                              key="start",
                              value="/bin/foo")
        self.set_test_systemd(self.default_appname,
                              key="stop",
                              value="bin/bar")
        self.set_test_systemd(self.default_appname,
                              key="nonexistent",
                              value="foo")
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_unknown()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 1, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_description(self):
        '''Test check_snappy_service_description()'''
        self._set_service([("description", "some description")])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_description()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_description_unspecified(self):
        '''Test check_snappy_service_description() - unspecified'''
        # self._set_service([("description", None)])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_description()
        r = c.click_report
        # required check is done elsewhere, so no error
        expected_counts = {'info': 0, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_description_empty(self):
        '''Test check_snappy_service_description() - empty'''
        self._set_service([("description", "")])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_description()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_start(self):
        '''Test check_snappy_service_start()'''
        self._set_service([("start", "some/start")])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_start()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_start_unspecified(self):
        '''Test check_snappy_service_start() - unspecified'''
        # self._set_service([("start", None)])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_start()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_start_empty(self):
        '''Test check_snappy_service_start() - empty'''
        self._set_service([("start", "")])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_start()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_start_absolute_path(self):
        '''Test check_snappy_service_start() - absolute path'''
        self._set_service([("start", "/foo/bar/some/start")])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_start()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_stop(self):
        '''Test check_snappy_service_stop()'''
        self._set_service([("stop", "some/stop")])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_stop()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_stop_unspecified(self):
        '''Test check_snappy_service_stop() - unspecified'''
        # self._set_service([("stop", None)])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_stop()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_stop_empty(self):
        '''Test check_snappy_service_stop() - empty'''
        self._set_service([("stop", "")])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_stop()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_stop_absolute_path(self):
        '''Test check_snappy_service_stop() - absolute path'''
        self._set_service([("stop", "/foo/bar/some/stop")])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_stop()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_poststop(self):
        '''Test check_snappy_service_poststop()'''
        self._set_service([("poststop", "some/poststop")])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_poststop()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_poststop_unspecified(self):
        '''Test check_snappy_service_poststop() - unspecified'''
        # self._set_service([("poststop", None)])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_poststop()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_poststop_empty(self):
        '''Test check_snappy_service_poststop() - empty'''
        self._set_service([("poststop", "")])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_poststop()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_poststop_absolute_path(self):
        '''Test check_snappy_service_poststop() - absolute path'''
        self._set_service([("poststop", "/foo/bar/some/poststop")])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_poststop()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_stop_timeout(self):
        '''Test check_snappy_service_stop_timeout()'''
        self._set_service([("start", "bin/foo"),
                           ("description", "something"),
                           ("stop-timeout", 30)])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_stop_timeout()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_stop_timeout_granularity(self):
        '''Test check_snappy_service_stop_timeout()'''
        self._set_service([("start", "bin/foo"),
                           ("description", "something"),
                           ("stop-timeout", '30s')])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_stop_timeout()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_stop_timeout_empty(self):
        '''Test check_snappy_service_stop_timeout() - empty'''
        self._set_service([("start", "bin/foo"),
                           ("description", "something"),
                           ("stop-timeout", "")])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_stop_timeout()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_stop_timeout_bad(self):
        '''Test check_snappy_service_stop_timeout() - bad'''
        self._set_service([("start", "bin/foo"),
                           ("description", "something"),
                           ("stop-timeout", "a")])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_stop_timeout()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_stop_timeout_bad_granularity(self):
        '''Test check_snappy_service_stop_timeout() - bad with granularity'''
        self._set_service([("start", "bin/foo"),
                           ("description", "something"),
                           ("stop-timeout", "30a")])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_stop_timeout()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_stop_timeout_range_low(self):
        '''Test check_snappy_service_stop_timeout() - out of range (low)'''
        self._set_service([("start", "bin/foo"),
                           ("description", "something"),
                           ("stop-timeout", -1)])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_stop_timeout()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_stop_timeout_range_high(self):
        '''Test check_snappy_service_stop_timeout() - out of range (high)'''
        self._set_service([("start", "bin/foo"),
                           ("description", "something"),
                           ("stop-timeout", 61)])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_stop_timeout()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_bus_name_pkgname(self):
        '''Test check_snappy_service_bus_name() - pkgname'''
        name = self.test_name.split('_')[0]
        self.set_test_pkg_yaml("name", name)
        self._set_service([("start", "bin/test-app"),
                           ("description", "something"),
                           ("bus-name", name)])
        self.set_test_pkg_yaml("type", 'framework')
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_bus_name()
        r = c.click_report
        expected_counts = {'info': 4, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_bus_name_appname(self):
        '''Test check_snappy_service_bus_name() - appname'''
        name = self.test_name.split('_')[0]
        self._set_service([("start", "bin/test-app"),
                           ("description", "something"),
                           ("bus-name", "%s.%s" % (name, "test-app"))])
        self.set_test_pkg_yaml("type", 'framework')
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_bus_name()
        r = c.click_report
        expected_counts = {'info': 4, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_bus_name_missing_framework_app(self):
        '''Test check_snappy_service_bus_name() - missing framework (app)'''
        name = self.test_name.split('_')[0]
        self._set_service([("start", "bin/test-app"),
                           ("description", "something"),
                           ("bus-name", "%s.%s" % (name, "test-app"))])
        self.set_test_pkg_yaml("type", 'app')
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_bus_name()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_bus_name_missing_framework_oem(self):
        '''Test check_snappy_service_bus_name() - missing framework (oem)'''
        name = self.test_name.split('_')[0]
        self._set_service([("start", "bin/test-app"),
                           ("description", "something"),
                           ("bus-name", "%s.%s" % (name, "test-app"))])
        self.set_test_pkg_yaml("type", 'oem')
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_bus_name()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_bus_name_pkgname_bad(self):
        '''Test check_snappy_service_bus_name() - bad pkgname'''
        name = self.test_name.split('_')[0]
        self.set_test_pkg_yaml("name", name)
        self._set_service([("start", "bin/test-app"),
                           ("description", "something"),
                           ("bus-name", name + "-bad")])
        self.set_test_pkg_yaml("type", 'framework')
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_bus_name()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_bus_name_appname_bad(self):
        '''Test check_snappy_service_bus_name() - bad appname'''
        name = self.test_name.split('_')[0]
        self._set_service([("start", "bin/test-app"),
                           ("description", "something"),
                           ("bus-name", "%s.%s-bad" % (name, "test-app"))])
        self.set_test_pkg_yaml("type", 'framework')
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_bus_name()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_bus_name_empty(self):
        '''Test check_snappy_service_bus_name() - bad (empty)'''
        name = self.test_name.split('_')[0]
        self.set_test_pkg_yaml("name", name)
        self._set_service([("start", "bin/test-app"),
                           ("description", "something"),
                           ("bus-name", "")])
        self.set_test_pkg_yaml("type", 'framework')
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_bus_name()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_bus_name_bad_regex(self):
        '''Test check_snappy_service_bus_name() - bad (regex)'''
        name = self.test_name.split('_')[0]
        self.set_test_pkg_yaml("name", name)
        self._set_service([("start", "bin/test-app"),
                           ("description", "something"),
                           ("bus-name", "name$")])
        self.set_test_pkg_yaml("type", 'framework')
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_bus_name()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 2}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_ports(self):
        '''Test check_snappy_service_ports()'''
        ports = self._create_ports()
        self.set_test_systemd(self.default_appname, "ports", ports)
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_ports()
        r = c.click_report
        expected_counts = {'info': 8, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_ports_internal(self):
        '''Test check_snappy_service_ports() - internal'''
        ports = self._create_ports()
        del ports['internal']
        self.set_test_systemd(self.default_appname, "ports", ports)
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_ports()
        r = c.click_report
        expected_counts = {'info': 6, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_ports_external(self):
        '''Test check_snappy_service_ports() - external'''
        ports = self._create_ports()
        del ports['external']
        self.set_test_systemd(self.default_appname, "ports", ports)
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_ports()
        r = c.click_report
        expected_counts = {'info': 4, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_ports_empty(self):
        '''Test check_snappy_service_ports() - empty'''
        ports = self._create_ports()
        del ports['internal']
        del ports['external']
        self.set_test_systemd(self.default_appname, "ports", ports)
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_ports()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_ports_bad_key(self):
        '''Test check_snappy_service_ports() - bad key'''
        ports = self._create_ports()
        ports['xternal'] = ports['external']
        del ports['external']

        self.set_test_systemd(self.default_appname, "ports", ports)
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_ports()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_ports_missing_internal(self):
        '''Test check_snappy_service_ports() - missing internal'''
        ports = self._create_ports()
        del ports['internal']['int1']

        self.set_test_systemd(self.default_appname, "ports", ports)
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_ports()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_ports_missing_external(self):
        '''Test check_snappy_service_ports() - missing external'''
        ports = self._create_ports()
        del ports['external']['ext1']
        del ports['external']['ext2']

        self.set_test_systemd(self.default_appname, "ports", ports)
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_ports()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_ports_missing_external_subkey(self):
        '''Test check_snappy_service_ports() - missing external subkey'''
        ports = self._create_ports()
        del ports['external']['ext2']['port']

        self.set_test_systemd(self.default_appname, "ports", ports)
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_ports()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_ports_invalid_internal_subkey(self):
        '''Test check_snappy_service_ports() - invalid internal subkey'''
        ports = self._create_ports()
        ports['internal']['int1']['prt'] = ports['internal']['int1']['port']
        del ports['internal']['int1']['port']
        del ports['internal']['int1']['negotiable']

        self.set_test_systemd(self.default_appname, "ports", ports)
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_ports()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_ports_invalid_internal_port(self):
        '''Test check_snappy_service_ports() - invalid internal port'''
        ports = self._create_ports()
        ports['internal']['int1']['port'] = "bad/8080"

        self.set_test_systemd(self.default_appname, "ports", ports)
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_ports()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_ports_invalid_internal_low_port(self):
        '''Test check_snappy_service_ports() - invalid internal low port'''
        ports = self._create_ports()
        ports['internal']['int1']['port'] = "0/tcp"

        self.set_test_systemd(self.default_appname, "ports", ports)
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_ports()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_ports_invalid_internal_high_port(self):
        '''Test check_snappy_service_ports() - invalid internal high port'''
        ports = self._create_ports()
        ports['internal']['int1']['port'] = "65536/tcp"

        self.set_test_systemd(self.default_appname, "ports", ports)
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_ports()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_ports_invalid_internal_negotiable(self):
        '''Test check_snappy_service_ports() - invalid internal negotiable'''
        ports = self._create_ports()
        ports['internal']['int1']['negotiable'] = -99999999

        self.set_test_systemd(self.default_appname, "ports", ports)
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_ports()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_listen_stream_abspkgname(self):
        '''Test check_snappy_service_listen_stream() - @pkgname'''
        name = self.test_name.split('_')[0]
        self.set_test_pkg_yaml("name", name)
        self._set_service([("start", "bin/test-app"),
                           ("description", "something"),
                           ("listen-stream", '@%s' % name)])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_listen_stream()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_listen_stream_abspkgname2(self):
        '''Test check_snappy_service_listen_stream() - @pkgname_'''
        name = self.test_name.split('_')[0]
        self.set_test_pkg_yaml("name", name)
        self._set_service([("start", "bin/test-app"),
                           ("description", "something"),
                           ("listen-stream", '@%s_something' % name)])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_listen_stream()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_listen_stream_bad_abstract(self):
        '''Test check_snappy_service_listen_stream() - bad (wrong name)'''
        name = self.test_name.split('_')[0]
        self.set_test_pkg_yaml("name", name)
        self._set_service([("start", "bin/test-app"),
                           ("description", "something"),
                           ("listen-stream", '@%s/nomatch' % name)])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_listen_stream()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_listen_stream_bad_relative(self):
        '''Test check_snappy_service_listen_stream() - bad (not / or @)'''
        name = self.test_name.split('_')[0]
        self.set_test_pkg_yaml("name", name)
        self._set_service([("start", "bin/test-app"),
                           ("description", "something"),
                           ("listen-stream", name)])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_listen_stream()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_listen_stream_empty(self):
        '''Test check_snappy_service_listen_stream() - empty'''
        name = self.test_name.split('_')[0]
        self.set_test_pkg_yaml("name", name)
        self._set_service([("start", "bin/test-app"),
                           ("description", "something"),
                           ("listen-stream", "")])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_listen_stream()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_socket_user(self):
        '''Test check_snappy_service_socket_user()'''
        name = self.test_name.split('_')[0]
        self.set_test_pkg_yaml("name", name)
        self._set_service([("start", "bin/test-app"),
                           ("description", "something"),
                           ("listen-stream", '@%s' % name),
                           ("socket-user", name)])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_socket_user()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_socket_user_no_listen_stream(self):
        '''Test check_snappy_service_socket_user() - missing listen-stream'''
        name = self.test_name.split('_')[0]
        self.set_test_pkg_yaml("name", name)
        self._set_service([("start", "bin/test-app"),
                           ("description", "something"),
                           ("socket-user", name)])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_socket_user()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 2}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_socket_user_bad(self):
        '''Test check_snappy_service_socket_user() - bad user'''
        name = self.test_name.split('_')[0]
        self.set_test_pkg_yaml("name", name)
        self._set_service([("start", "bin/test-app"),
                           ("description", "something"),
                           ("listen-stream", '@%s' % name),
                           ("socket-user", name + "nomatch")])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_socket_user()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 2}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_socket_group(self):
        '''Test check_snappy_service_socket_group()'''
        name = self.test_name.split('_')[0]
        self.set_test_pkg_yaml("name", name)
        self._set_service([("start", "bin/test-app"),
                           ("description", "something"),
                           ("listen-stream", '@%s' % name),
                           ("socket-group", name)])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_socket_group()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_socket_group_no_listen_stream(self):
        '''Test check_snappy_service_socket_group() - missing listen-stream'''
        name = self.test_name.split('_')[0]
        self.set_test_pkg_yaml("name", name)
        self._set_service([("start", "bin/test-app"),
                           ("description", "something"),
                           ("socket-group", name)])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_socket_group()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 2}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_socket_group_bad(self):
        '''Test check_snappy_service_socket_group() - bad group'''
        name = self.test_name.split('_')[0]
        self.set_test_pkg_yaml("name", name)
        self._set_service([("start", "bin/test-app"),
                           ("description", "something"),
                           ("listen-stream", '@%s' % name),
                           ("socket-group", name + "nomatch")])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_socket_group()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 2}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_socket(self):
        '''Test check_snappy_service_socket()'''
        name = self.test_name.split('_')[0]
        self.set_test_pkg_yaml("name", name)
        self._set_service([("start", "bin/test-app"),
                           ("description", "something"),
                           ("listen-stream", '@%s' % name),
                           ("socket", True)])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_socket()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_snappy_service_socket_no_listen_stream(self):
        '''Test check_snappy_service_socket() - missing listen-stream'''
        name = self.test_name.split('_')[0]
        self.set_test_pkg_yaml("name", name)
        self._set_service([("start", "bin/test-app"),
                           ("description", "something"),
                           ("socket", True)])
        c = ClickReviewSystemd(self.test_name)
        c.check_snappy_service_socket()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)



'''cr_tests.py: common setup and tests for test modules'''
#
# Copyright (C) 2013-2015 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import io
import json
import os
import tempfile
from xdg.DesktopEntry import DesktopEntry
import yaml

from unittest.mock import patch
from unittest import TestCase

from clickreviews.cr_lint import MINIMUM_CLICK_FRAMEWORK_VERSION
import clickreviews.common as common

# These should be set in the test cases
TEST_CONTROL = ""
TEST_MANIFEST = ""
TEST_PKG_YAML = ""
TEST_HASHES_YAML = ""
TEST_README_MD = ""
TEST_SECURITY = dict()
TEST_SECURITY_PROFILES = dict()
TEST_DESKTOP = dict()
TEST_WEBAPP_MANIFESTS = dict()
TEST_URLS = dict()
TEST_SCOPES = dict()
TEST_CONTENT_HUB = dict()
TEST_ACCOUNTS_MANIFEST = dict()
TEST_ACCOUNTS_APPLICATION = dict()
TEST_ACCOUNTS_PROVIDER = dict()
TEST_ACCOUNTS_QML_PLUGIN = dict()
TEST_ACCOUNTS_SERVICE = dict()
TEST_PUSH_HELPER = dict()
TEST_BIN_PATH = dict()
TEST_FRAMEWORK = dict()
TEST_FRAMEWORK_POLICY = dict()
TEST_FRAMEWORK_POLICY_UNKNOWN = []
TEST_PKGFMT_TYPE = "click"
TEST_PKGFMT_VERSION = "0.4"


#
# Mock override functions
#
def _mock_func(self):
    '''Fake test function'''
    return


def _extract_control_file(self):
    '''Pretend we read the control file'''
    return io.StringIO(TEST_CONTROL)


def _extract_manifest_file(self):
    '''Pretend we read the manifest file'''
    return io.StringIO(TEST_MANIFEST)


def _extract_package_yaml(self):
    '''Pretend we read the package.yaml file'''
    return io.StringIO(TEST_PKG_YAML)


def _extract_hashes_yaml(self):
    '''Pretend we read the hashes.yaml file'''
    return io.StringIO(TEST_HASHES_YAML)


def _path_join(self, d, fn):
    '''Pretend we have a tempdir'''
    return os.path.join("/fake", fn)


def _get_sha512sum(self, fn):
    '''Pretend we found performed a sha512'''
    (rc, out) = common.cmd(['sha512sum', os.path.realpath(__file__)])
    if rc != 0:
        return None
    return out.split()[0]


def _extract_statinfo(self, fn):
    '''Pretend we found performed an os.stat()'''
    return os.stat(os.path.realpath(__file__))


def _extract_readme_md(self):
    '''Pretend we read the meta/readme.md file'''
    return TEST_README_MD


def _check_innerpath_executable(self, fn):
    '''Pretend we a file'''
    if '.nonexec' in fn:
        return False
    return True


def _extract_click_frameworks(self):
    '''Pretend we enumerated the click frameworks'''
    return ["ubuntu-sdk-13.10",
            "ubuntu-sdk-14.04-dev1",
            "ubuntu-sdk-14.04-html-dev1",
            "ubuntu-sdk-14.04-papi-dev1",
            "ubuntu-sdk-14.04-qml-dev1",
            "ubuntu-sdk-14.04",
            "ubuntu-sdk-14.04-html",
            "ubuntu-sdk-14.04-papi",
            "ubuntu-sdk-14.04-qml",
            "ubuntu-sdk-14.10-dev1",
            "ubuntu-sdk-14.10-html-dev1",
            "ubuntu-sdk-14.10-papi-dev1",
            "ubuntu-sdk-14.10-qml-dev1",
            "ubuntu-sdk-14.10-dev2",
            "ubuntu-sdk-14.10-html-dev2",
            "ubuntu-sdk-14.10-papi-dev2",
            "ubuntu-sdk-14.10-qml-dev2",
            ]


def _extract_security_manifest(self, app):
    '''Pretend we read the security manifest file'''
    return io.StringIO(TEST_SECURITY[app])


def _get_security_manifest(self, app):
    '''Pretend we read the security manifest file'''
    return ("%s.apparmor" % app, json.loads(TEST_SECURITY[app]))


def _extract_security_profile(self, app):
    '''Pretend we read the security profile'''
    return io.StringIO(TEST_SECURITY_PROFILES[app])


def _get_security_profile(self, app):
    '''Pretend we read the security profile'''
    return ("%s.profile" % app, TEST_SECURITY_PROFILES[app])


def _get_security_supported_policy_versions(self):
    '''Pretend we read the contens of /usr/share/apparmor/easyprof'''
    return [1.0, 1.1, 1.2, 1.3]


def _extract_desktop_entry(self, app):
    '''Pretend we read the desktop file'''
    return ("%s.desktop" % app, TEST_DESKTOP[app])


def _get_desktop_entry(self, app):
    '''Pretend we read the desktop file'''
    return TEST_DESKTOP[app]


def _extract_webapp_manifests(self):
    '''Pretend we read the webapp manifest files'''
    return TEST_WEBAPP_MANIFESTS


def _extract_url_dispatcher(self, app):
    '''Pretend we read the url dispatcher file'''
    return ("%s.url-dispatcher" % app, TEST_URLS[app])


def _extract_scopes(self, app):
    '''Pretend we found and read the files in the scope directories'''
    return TEST_SCOPES[app]


def _extract_content_hub(self, app):
    '''Pretend we read the content-hub file'''
    return ("%s.content.json" % app, TEST_CONTENT_HUB[app])


def _extract_account(self, app, account_type):
    '''Pretend we read the accounts file'''
    f = app
    val = None
    if account_type == "accounts":
        f += ".accounts"
        val = TEST_ACCOUNTS_MANIFEST[app]
    elif account_type == "account-application":
        f += ".application"
        val = TEST_ACCOUNTS_APPLICATION[app]
    elif account_type == "account-provider":
        f += ".provider"
        val = TEST_ACCOUNTS_PROVIDER[app]
    elif account_type == "account-qml-plugin":
        f += ".qml-plugin"
        val = TEST_ACCOUNTS_QML_PLUGIN[app]
    elif account_type == "account-service":
        f += ".service"
        val = TEST_ACCOUNTS_SERVICE[app]
    else:  # should never get here
        raise ValueError("Unknown account_type '%s'" % account_type)

    return (f, val)


def _extract_push_helper(self, app):
    '''Pretend we read the push-helper file'''
    return ("%s.push-helper.json" % app, TEST_PUSH_HELPER[app])


def _extract_bin_path(self, app):
    '''Pretend we found the bin-path file'''
    return ("%s" % TEST_BIN_PATH[app])


def _check_bin_path_executable(self, app):
    '''Pretend we found the bin-path file'''
    if TEST_BIN_PATH[app].endswith('.nonexec'):
        return False
    return True


def _extract_framework(self, app):
    '''Pretend we found the framework file'''
    return ("%s.framework" % app, TEST_FRAMEWORK[app])


def _extract_framework_policy(self):
    '''Pretend we found the framework policy files'''
    return (TEST_FRAMEWORK_POLICY, TEST_FRAMEWORK_POLICY_UNKNOWN)


def _has_framework_in_metadir(self):
    '''Pretend we found the framework file'''
    return True


def _pkgfmt_type(self):
    '''Pretend we found the pkgfmt type'''
    return TEST_PKGFMT_TYPE


def _pkgfmt_version(self):
    '''Pretend we found the pkgfmt version'''
    return TEST_PKGFMT_VERSION


def _detect_package(self, fn):
    '''Pretend we detected the package'''
    ver = 1
    if TEST_PKGFMT_TYPE == "snap" and TEST_PKGFMT_VERSION != "15.04":
        ver = 2
    return (TEST_PKGFMT_TYPE, ver)


def create_patches():
    # http://docs.python.org/3.4/library/unittest.mock-examples.html
    # Mock patching. Don't use decorators but instead patch in setUp() of the
    # child.
    patches = []
    patches.append(patch('clickreviews.common.Review._check_package_exists',
                   _mock_func))
    patches.append(patch(
        'clickreviews.cr_common.ClickReview._extract_control_file',
        _extract_control_file))
    patches.append(patch(
        'clickreviews.cr_common.ClickReview._extract_manifest_file',
        _extract_manifest_file))
    patches.append(patch(
        'clickreviews.cr_common.ClickReview._extract_package_yaml',
        _extract_package_yaml))
    patches.append(patch(
        'clickreviews.cr_common.ClickReview._extract_hashes_yaml',
        _extract_hashes_yaml))
    patches.append(patch('clickreviews.common.Review._path_join', _path_join))
    patches.append(patch(
        'clickreviews.common.Review._get_sha512sum', _get_sha512sum))
    patches.append(patch(
        'clickreviews.common.Review._extract_statinfo', _extract_statinfo))
    patches.append(patch(
        'clickreviews.cr_common.ClickReview._extract_click_frameworks',
        _extract_click_frameworks))
    patches.append(patch('clickreviews.common.unpack_pkg', _mock_func))
    patches.append(patch('clickreviews.common.raw_unpack_pkg', _mock_func))
    patches.append(patch('clickreviews.common.detect_package',
                   _detect_package))
    patches.append(patch('clickreviews.common.Review._list_all_files',
                   _mock_func))
    patches.append(patch(
        'clickreviews.common.Review._list_all_compiled_binaries', _mock_func))

    # lint overrides
    patches.append(patch(
                   'clickreviews.cr_lint.ClickReviewLint._list_control_files',
                   _mock_func))
    patches.append(patch(
        'clickreviews.cr_lint.ClickReviewLint._extract_readme_md',
        _extract_readme_md))
    patches.append(patch(
        'clickreviews.common.Review._check_innerpath_executable',
        _check_innerpath_executable))

    # security overrides
    patches.append(patch('clickreviews.cr_security.ClickReviewSecurity.'
                         '_extract_security_manifest',
                         _extract_security_manifest))
    patches.append(patch(
        'clickreviews.cr_security.ClickReviewSecurity._get_security_manifest',
        _get_security_manifest))
    patches.append(patch('clickreviews.cr_security.ClickReviewSecurity.'
                         '_extract_security_profile',
                         _extract_security_profile))
    patches.append(patch(
        'clickreviews.cr_security.ClickReviewSecurity._get_security_profile',
        _get_security_profile))

    # desktop overrides
    patches.append(patch(
        'clickreviews.cr_desktop.ClickReviewDesktop._extract_desktop_entry',
        _extract_desktop_entry))
    patches.append(patch(
        'clickreviews.cr_desktop.ClickReviewDesktop._get_desktop_entry',
        _get_desktop_entry))
    patches.append(patch(
        'clickreviews.cr_desktop.ClickReviewDesktop._extract_webapp_manifests',
        _extract_webapp_manifests))

    # url-dispatcher overrides
    patches.append(patch('clickreviews.cr_url_dispatcher.'
                         'ClickReviewUrlDispatcher._extract_url_dispatcher',
                         _extract_url_dispatcher))

    # scope overrides
    patches.append(patch(
        'clickreviews.cr_scope.ClickReviewScope._extract_scopes',
        _extract_scopes))

    # content-hub overrides
    patches.append(patch('clickreviews.cr_content_hub.ClickReviewContentHub.'
                         '_extract_content_hub',
                         _extract_content_hub))

    # online accounts overrides
    patches.append(patch(
        'clickreviews.cr_online_accounts.ClickReviewAccounts._extract_account',
        _extract_account))

    # push-helper overrides
    patches.append(patch('clickreviews.cr_push_helper.ClickReviewPushHelper.'
                         '_extract_push_helper',
                         _extract_push_helper))

    # bin-path overrides
    patches.append(patch(
        'clickreviews.cr_bin_path.ClickReviewBinPath._extract_bin_path',
        _extract_bin_path))
    patches.append(patch('clickreviews.cr_bin_path.ClickReviewBinPath.'
                         '_check_bin_path_executable',
                         _check_bin_path_executable))

    # framework overrides
    patches.append(patch(
        'clickreviews.cr_framework.ClickReviewFramework._extract_framework',
        _extract_framework))
    patches.append(patch('clickreviews.cr_framework.ClickReviewFramework.'
                         '_extract_framework_policy',
                         _extract_framework_policy))
    patches.append(patch('clickreviews.cr_framework.ClickReviewFramework.'
                         '_has_framework_in_metadir',
                         _has_framework_in_metadir))

    # pkgfmt
    patches.append(patch("clickreviews.common.Review._pkgfmt_type",
                   _pkgfmt_type))
    patches.append(patch("clickreviews.common.Review._pkgfmt_version",
                   _pkgfmt_version))

    return patches


class TestClickReview(TestCase):
    """Tests for the lint review tool."""
    def __init__(self, *args):
        if not hasattr(self, 'desktop_tmpdir'):
            self.desktop_tmpdir = \
                tempfile.mkdtemp(prefix="clickreview-test-desktop-")
        TestCase.__init__(self, *args)
        self._reset_test_data()

    def _reset_test_data(self):
        # dictionary representing DEBIAN/control
        self.test_control = dict()
        self.set_test_control('Package',
                              "com.ubuntu.developer.someuser.testapp")
        self.set_test_control('Version', "1.0")
        self.set_test_control('Click-Version', MINIMUM_CLICK_FRAMEWORK_VERSION)
        self.set_test_control('Architecture', "all")
        self.set_test_control('Maintainer',
                              "Some User <some.user@example.com>")
        self.set_test_control('Installed-Size', "111")
        self.set_test_control('Description', "My Test App")

        # dictionary representing DEBIAN/manifest
        self.test_manifest = dict()
        self.set_test_manifest("description",
                               "Some longish description of My Test App")
        self.set_test_manifest("framework", "ubuntu-sdk-13.10")
        self.set_test_manifest("maintainer", self.test_control['Maintainer'])
        self.set_test_manifest("name", self.test_control['Package'])
        self.set_test_manifest("title", self.test_control['Description'])
        self.set_test_manifest("version", self.test_control['Version'])
        self.test_manifest["hooks"] = dict()
        self.default_appname = "test-app"
        self.test_manifest["hooks"][self.default_appname] = dict()
        self.test_manifest["hooks"][self.default_appname]["apparmor"] = \
            "%s.apparmor" % self.default_appname
        self.test_manifest["hooks"][self.default_appname]["desktop"] = \
            "%s.desktop" % self.default_appname
        self.test_manifest["hooks"][self.default_appname]["urls"] = \
            "%s.url-dispatcher" % self.default_appname
        self._update_test_manifest()

        self.test_pkg_yaml = dict()
        self.set_test_pkg_yaml("name", self.test_control['Package'])
        self.set_test_pkg_yaml("version", self.test_control['Version'])
        self.set_test_pkg_yaml("architectures",
                               [self.test_control['Architecture']])
        self._update_test_pkg_yaml()

        self.test_hashes_yaml = dict()
        self._update_test_hashes_yaml()

        self.test_readme_md = self.test_control['Description']
        self._update_test_readme_md()

        self.set_test_pkgfmt("click", "0.4")

        # hooks
        self.test_security_manifests = dict()
        self.test_security_profiles = dict()
        self.test_desktop_files = dict()
        self.test_url_dispatcher = dict()
        self.test_scopes = dict()
        self.test_content_hub = dict()
        self.test_accounts_manifest = dict()
        self.test_accounts_application = dict()
        self.test_accounts_provider = dict()
        self.test_accounts_qml_plugin = dict()
        self.test_accounts_service = dict()
        self.test_push_helper = dict()
        self.test_bin_path = dict()
        self.test_framework = dict()
        self.test_framework_policy = dict()
        self.test_framework_policy_unknown = []

        for app in self.test_manifest["hooks"].keys():
            # setup security manifest for each app
            self.set_test_security_manifest(app, 'policy_groups',
                                            ['networking'])
            self.set_test_security_manifest(app, 'policy_version', 1.0)

            # setup desktop file for each app
            self.set_test_desktop(app, 'Name',
                                  self.default_appname,
                                  no_update=True)
            self.set_test_desktop(app, 'Comment', '%s test comment' % app,
                                  no_update=True)
            self.set_test_desktop(app, 'Exec', 'qmlscene %s.qml' % app,
                                  no_update=True)
            self.set_test_desktop(app, 'Icon', '%s.png' % app, no_update=True)
            self.set_test_desktop(app, 'Terminal', 'false', no_update=True)
            self.set_test_desktop(app, 'Type', 'Application', no_update=True)
            self.set_test_desktop(app, 'X-Ubuntu-Touch', 'true',
                                  no_update=True)

            self.set_test_url_dispatcher(app, None, None)

            # Ensure we have no scope entries since they conflict with desktop.
            # Scope tests will have to add them as part of their tests.
            self.set_test_scope(app, None)

            # Reset to no content-hub entries in manifest
            self.set_test_content_hub(app, None, None)

            # Reset to no accounts entries in manifest
            self.set_test_account(app, "accounts", None)
            self.set_test_account(app, "account-application", None)
            self.set_test_account(app, "account-provider", None)
            self.set_test_account(app, "account-qml-plugin", None)
            self.set_test_account(app, "account-service", None)

            # Reset to no push-helper entries in manifest
            self.set_test_push_helper(app, None, None)

            # Reset to no bin-path entries in manifest
            self.set_test_bin_path(app, None)

            # Reset to no framework entries in manifest
            self.set_test_framework(app, None, None)

            # Reset to no framework entries in manifest
            self.set_test_framework_policy(None)
            self.set_test_framework_policy_unknown([])

            # Reset to no security profiles
            self.set_test_security_profile(app, None)

        self._update_test_security_manifests()
        self._update_test_security_profiles()
        self._update_test_desktop_files()
        self._update_test_url_dispatcher()
        self._update_test_scopes()
        self._update_test_content_hub()
        self._update_test_accounts_manifest()
        self._update_test_accounts_application()
        self._update_test_accounts_provider()
        self._update_test_accounts_qml_plugin()
        self._update_test_accounts_service()
        self._update_test_push_helper()
        self._update_test_bin_path()
        self._update_test_framework()
        self._update_test_framework_policy()
        self._update_test_framework_policy_unknown()

        # webapps manifests (leave empty for now)
        self.test_webapp_manifests = dict()
        self._update_test_webapp_manifests()

        # mockup a click package name based on the above
        self._update_test_name()

    def _update_test_control(self):
        global TEST_CONTROL
        TEST_CONTROL = ""
        for k in self.test_control.keys():
            TEST_CONTROL += "%s: %s\n" % (k, self.test_control[k])

    def _update_test_manifest(self):
        global TEST_MANIFEST
        TEST_MANIFEST = json.dumps(self.test_manifest)

    def _update_test_pkg_yaml(self):
        global TEST_PKG_YAML
        TEST_PKG_YAML = yaml.dump(self.test_pkg_yaml,
                                  default_flow_style=False,
                                  indent=4)

    def _update_test_hashes_yaml(self):
        global TEST_HASHES_YAML
        TEST_HASHES_YAML = yaml.dump(self.test_hashes_yaml,
                                     default_flow_style=False,
                                     indent=4)

    def _update_test_readme_md(self):
        global TEST_README_MD
        TEST_README_MD = self.test_readme_md

    def _update_test_security_manifests(self):
        global TEST_SECURITY
        TEST_SECURITY = dict()
        for app in self.test_security_manifests.keys():
            TEST_SECURITY[app] = json.dumps(self.test_security_manifests[app])

    def _update_test_security_profiles(self):
        global TEST_SECURITY_PROFILES
        TEST_SECURITY_PROFILES = dict()
        for app in self.test_security_profiles.keys():
            TEST_SECURITY_PROFILES[app] = self.test_security_profiles[app]
            self.test_manifest["hooks"][app]["apparmor-profile"] = \
                "%s.profile" % app
        self._update_test_manifest()

    def _update_test_desktop_files(self):
        global TEST_DESKTOP
        TEST_DESKTOP = dict()
        for app in self.test_desktop_files.keys():
            contents = '''[Desktop Entry]'''
            for k in self.test_desktop_files[app].keys():
                contents += '\n%s=%s' % (k, self.test_desktop_files[app][k])
            contents += "\n"

            fn = os.path.join(self.desktop_tmpdir, "%s.desktop" % app)
            with open(fn, "w") as f:
                f.write(contents)
            f.close()
            TEST_DESKTOP[app] = DesktopEntry(fn)

    def _update_test_webapp_manifests(self):
        global TEST_WEBAPP_MANIFESTS
        TEST_WEBAPP_MANIFESTS = dict()
        for i in self.test_webapp_manifests.keys():
            TEST_WEBAPP_MANIFESTS[i] = self.test_webapp_manifests[i]

    def _update_test_url_dispatcher(self):
        global TEST_URLS
        TEST_URLS = dict()
        for app in self.test_url_dispatcher.keys():
            TEST_URLS[app] = self.test_url_dispatcher[app]

    def _update_test_scopes(self):
        global TEST_SCOPES
        TEST_SCOPES = dict()
        for app in self.test_scopes.keys():
            TEST_SCOPES[app] = self.test_scopes[app]
            self.test_manifest["hooks"][app]["scope"] = \
                TEST_SCOPES[app]["dir_rel"]
        self._update_test_manifest()

    def _update_test_content_hub(self):
        global TEST_CONTENT_HUB
        TEST_CONTENT_HUB = dict()
        for app in self.test_content_hub.keys():
            TEST_CONTENT_HUB[app] = self.test_content_hub[app]
            self.test_manifest["hooks"][app]["content-hub"] = \
                "%s.content.json" % app
        self._update_test_manifest()

    def _update_test_accounts_manifest(self):
        global TEST_ACCOUNTS_MANIFEST
        TEST_ACCOUNTS_MANIFEST = dict()
        for app in self.test_accounts_manifest.keys():
            TEST_ACCOUNTS_MANIFEST[app] = self.test_accounts_manifest[app]
            self.test_manifest["hooks"][app]["accounts"] = \
                "%s.accounts" % app
        self._update_test_manifest()

    def _update_test_accounts_application(self):
        global TEST_ACCOUNTS_APPLICATION
        TEST_ACCOUNTS_APPLICATION = dict()
        for app in self.test_accounts_application.keys():
            TEST_ACCOUNTS_APPLICATION[app] = \
                self.test_accounts_application[app]
            self.test_manifest["hooks"][app]["account-application"] = \
                "%s.application" % app
        self._update_test_manifest()

    def _update_test_accounts_provider(self):
        global TEST_ACCOUNTS_PROVIDER
        TEST_ACCOUNTS_PROVIDER = dict()
        for app in self.test_accounts_provider.keys():
            TEST_ACCOUNTS_PROVIDER[app] = self.test_accounts_provider[app]
            self.test_manifest["hooks"][app]["account-provider"] = \
                "%s.provider" % app
        self._update_test_manifest()

    def _update_test_accounts_qml_plugin(self):
        global TEST_ACCOUNTS_QML_PLUGIN
        TEST_ACCOUNTS_QML_PLUGIN = dict()
        for app in self.test_accounts_qml_plugin.keys():
            TEST_ACCOUNTS_QML_PLUGIN[app] = self.test_accounts_qml_plugin[app]
            self.test_manifest["hooks"][app]["account-qml-plugin"] = \
                "%s.qml_plugin" % app
        self._update_test_manifest()

    def _update_test_accounts_service(self):
        global TEST_ACCOUNTS_SERVICE
        TEST_ACCOUNTS_SERVICE = dict()
        for app in self.test_accounts_service.keys():
            TEST_ACCOUNTS_SERVICE[app] = self.test_accounts_service[app]
            self.test_manifest["hooks"][app]["account-service"] = \
                "%s.service" % app
        self._update_test_manifest()

    def _update_test_push_helper(self):
        global TEST_PUSH_HELPER
        TEST_PUSH_HELPER = dict()
        for app in self.test_push_helper.keys():
            TEST_PUSH_HELPER[app] = self.test_push_helper[app]
            self.test_manifest["hooks"][app]["push-helper"] = \
                "%s.push-helper.json" % app
        self._update_test_manifest()

    def _update_test_bin_path(self):
        global TEST_BIN_PATH
        TEST_BIN_PATH = dict()
        for app in self.test_bin_path.keys():
            TEST_BIN_PATH[app] = self.test_bin_path[app]
            self.test_manifest["hooks"][app]["bin-path"] = \
                "%s" % TEST_BIN_PATH[app]
        self._update_test_manifest()

    def _update_test_framework(self):
        global TEST_FRAMEWORK
        TEST_FRAMEWORK = dict()
        for app in self.test_framework.keys():
            TEST_FRAMEWORK[app] = self.test_framework[app]
            if app not in self.test_manifest["hooks"]:
                self.test_manifest["hooks"][app] = dict()
            self.test_manifest["hooks"][app]["framework"] = \
                "%s.framework" % TEST_FRAMEWORK[app]
        self._update_test_manifest()

    def _update_test_framework_policy(self):
        global TEST_FRAMEWORK_POLICY
        TEST_FRAMEWORK_POLICY = self.test_framework_policy

    def _update_test_framework_policy_unknown(self):
        global TEST_FRAMEWORK_POLICY_UNKNOWN
        TEST_FRAMEWORK_POLICY_UNKNOWN = self.test_framework_policy_unknown

    def _update_test_name(self):
        self.test_name = "%s_%s_%s.click" % (self.test_control['Package'],
                                             self.test_control['Version'],
                                             self.test_control['Architecture'])

    def check_results(self, report,
                      expected_counts={'info': 1, 'warn': 0, 'error': 0},
                      expected=None):
        common.check_results(self, report, expected_counts, expected)

    def check_manual_review(self, report, check_name,
                            result_type='error', manual_review=True):
        result = report[result_type][check_name]
        self.assertEqual(result['manual_review'], manual_review)

    def set_test_control(self, key, value):
        '''Set key in DEBIAN/control to value. If value is None, remove key'''
        if value is None:
            if key in self.test_control:
                self.test_control.pop(key, None)
        else:
            self.test_control[key] = value
        self._update_test_control()

    def set_test_manifest(self, key, value):
        '''Set key in DEBIAN/manifest to value. If value is None, remove key'''
        if value is None:
            if key in self.test_manifest:
                self.test_manifest.pop(key, None)
        else:
            self.test_manifest[key] = value
        self._update_test_manifest()

    def set_test_pkg_yaml(self, key, value):
        '''Set key in meta/package.yaml to value. If value is None, remove
           key'''
        if value is None:
            if key in self.test_pkg_yaml:
                self.test_pkg_yaml.pop(key, None)
        else:
            self.test_pkg_yaml[key] = value
        self._update_test_pkg_yaml()

    def set_test_hashes_yaml(self, yaml):
        '''Set hashes.yaml to yaml'''
        self.test_hashes_yaml = yaml
        self._update_test_hashes_yaml()

    def set_test_readme_md(self, contents):
        '''Set contents of meta/readme.md'''
        if contents is None:
            self.test_readme_md = None
        else:
            self.test_readme_md = contents
        self._update_test_readme_md()

    def set_test_security_manifest(self, app, key, value):
        '''Set key in security manifest and package.yaml to value. If value is
           None, remove key, if key is None, remove app.
        '''
        # package.yaml - we don't know if it is a service or a binary with
        # these values, so just use 'binaries'
        if key is None:
            if 'binaries' in self.test_pkg_yaml:
                for b in self.test_pkg_yaml['binaries']:
                    if 'name' in b and b['name'] == app:
                        self.test_pkg_yaml['binaries'].remove(b)
                        break
        elif value is None:
            if 'binaries' in self.test_pkg_yaml:
                found = False
                for b in self.test_pkg_yaml['binaries']:
                    if 'name' in b and b['name'] == app:
                        if key in b:
                            b.remove(key)
                            found = True
                            break
        else:
            found = False
            k = key
            if key == 'template':
                k = 'security-template'
            elif key == 'policy_groups':
                k = 'caps'
            if 'binaries' in self.test_pkg_yaml:
                for b in self.test_pkg_yaml['binaries']:
                    if 'name' in b and b['name'] == app:
                        # Found the entry, so update key/value
                        b[k] = value
                        found = True
                        break
            # Did not find the entry, so create one
            if not found:
                if 'binaries' not in self.test_pkg_yaml:
                    self.test_pkg_yaml['binaries'] = []
                self.test_pkg_yaml['binaries'].append({'name': app,
                                                       k: value})
        self._update_test_pkg_yaml()

        # click manifest
        if app not in self.test_security_manifests:
            self.test_security_manifests[app] = dict()

        if key is None:
            if app in self.test_security_manifests:
                del self.test_security_manifests[app]
                del self.test_manifest["hooks"][app]
        elif value is None:
            if key in self.test_security_manifests[app]:
                self.test_security_manifests[app].pop(key, None)
        else:
            self.test_security_manifests[app][key] = value
        self._update_test_security_manifests()

    def set_test_security_profile(self, app, policy):
        '''Set policy in security profile'''
        if policy is None:
            if app in self.test_security_profiles:
                self.test_security_profiles.pop(app)
        else:
            if app not in self.test_security_profiles:
                self.test_security_profiles[app] = dict()
            self.test_security_profiles[app] = policy
        self._update_test_security_profiles()

    def set_test_pkgfmt(self, t, v):
        global TEST_PKGFMT_TYPE
        global TEST_PKGFMT_VERSION
        TEST_PKGFMT_TYPE = t
        TEST_PKGFMT_VERSION = v

    def set_test_desktop(self, app, key, value, no_update=False):
        '''Set key in desktop file to value. If value is None, remove key'''
        if app not in self.test_desktop_files:
            self.test_desktop_files[app] = dict()

        if value is None:
            if key in self.test_desktop_files[app]:
                self.test_desktop_files[app].pop(key, None)
        else:
            self.test_desktop_files[app][key] = value
        if not no_update:
            self._update_test_desktop_files()

    def set_test_webapp_manifest(self, fn, key, value):
        '''Set key in webapp manifest to value. If value is None, remove
           key'''
        if key is None and value is None:
            self.test_webapp_manifests[fn] = None
            self._update_test_webapp_manifests()
            return

        if fn not in self.test_webapp_manifests:
            self.test_webapp_manifests[fn] = dict()

        if value is None:
            if key in self.test_webapp_manifests[fn]:
                self.test_webapp_manifests[fn].pop(key, None)
        else:
            self.test_webapp_manifests[fn][key] = value
        self._update_test_webapp_manifests()

    def set_test_url_dispatcher(self, app, key, value, append=False):
        '''Set url-dispatcher entries. If value is None, remove'''
        if app not in self.test_url_dispatcher:
            self.test_url_dispatcher[app] = []

        if value is None:
            self.test_url_dispatcher[app] = []
        else:
            if not append:
                self.test_url_dispatcher[app] = []
            self.test_url_dispatcher[app].append({key: value})
        self._update_test_url_dispatcher()

    def set_test_scope(self, app, scope):
        '''Set scope for app. If it is None, remove'''
        if scope is None:
            if app in self.test_scopes:
                self.test_scopes.pop(app)
            if 'scope' in self.test_manifest['hooks'][app]:
                self.test_manifest['hooks'][app].pop('scope', None)
        else:
            self.test_scopes[app] = scope
        self._update_test_scopes()

    def set_test_content_hub(self, app, key, value):
        '''Set content-hub entries. If value is None, remove key, if key is
           None, remove content-hub from manifest'''
        if key is None:
            if app in self.test_content_hub:
                self.test_content_hub.pop(app)
        elif value is None:
            if key in self.test_content_hub[app]:
                self.test_content_hub[app].pop(key)
        else:
            if app not in self.test_content_hub:
                self.test_content_hub[app] = dict()
            if key not in self.test_content_hub[app]:
                self.test_content_hub[app][key] = []
            self.test_content_hub[app][key].append(value)
        self._update_test_content_hub()

    def set_test_account(self, app, account_type, value):
        '''Set accounts XML. If value is None, remove from manifest'''
        if account_type == "accounts":
            d = self.test_accounts_manifest
        elif account_type == "account-application":
            d = self.test_accounts_application
        elif account_type == "account-provider":
            d = self.test_accounts_provider
        elif account_type == "account-qml-plugin":
            d = self.test_accounts_qml_plugin
        elif account_type == "account-service":
            d = self.test_accounts_service
        else:  # should never get here
            raise ValueError("Unknown account_type '%s'" % account_type)

        if value is None:
            if app in d:
                d[app] = None
        else:
            d[app] = value

        if account_type == "accounts":
            self._update_test_accounts_manifest()
        elif account_type == "account-application":
            self._update_test_accounts_application()
        elif account_type == "account-provider":
            self._update_test_accounts_provider()
        elif account_type == "account-qml-plugin":
            self._update_test_accounts_qml_plugin()
        elif account_type == "account-service":
            self._update_test_accounts_service()

    def set_test_push_helper(self, app, key, value):
        '''Set push-helper entries. If value is None, remove key, if key is
           None, remove push-helper from manifest'''
        if key is None:
            if app in self.test_push_helper:
                self.test_push_helper.pop(app)
        elif value is None:
            if key in self.test_push_helper[app]:
                self.test_push_helper[app].pop(key)
        else:
            if app not in self.test_push_helper:
                self.test_push_helper[app] = dict()
            self.test_push_helper[app][key] = value
        self._update_test_push_helper()

    def set_test_bin_path(self, app, value):
        '''Set bin-path entries. If value is None, remove bin-path from
           manifest and yaml. If app != value, set 'exec' in the yaml

           Note the click manifest and the package.yaml use different
           storage types. pkg_yaml['binaries'] is a list of dictionaries where
           manifest['hooks'] is a dictionary of dictionaries. This function
           sets the manifest entry and then a yaml entry with 'name' and 'exec'
           fields.

             manifest['hooks'][app]['bin-path'] = value
             pkg_yaml['binaries'][*]['name'] = app
             pkg_yaml['binaries'][*]['exec'] = value
        '''

        # Update the package.yaml
        if value is None:
            if 'binaries' in self.test_pkg_yaml:
                for b in self.test_pkg_yaml['binaries']:
                    if 'name' in b and b['name'] == app:
                        self.test_pkg_yaml['binaries'].remove(b)
                        break
        else:
            found = False
            if 'binaries' in self.test_pkg_yaml:
                for b in self.test_pkg_yaml['binaries']:
                    if 'name' in b and b['name'] == app:
                        found = True
                        break
            if not found:
                if 'binaries' not in self.test_pkg_yaml:
                    self.test_pkg_yaml['binaries'] = []
                if value == app:
                    self.test_pkg_yaml['binaries'].append({'name': app})
                else:
                    self.test_pkg_yaml['binaries'].append({'name': app,
                                                           'exec': value})
        self._update_test_pkg_yaml()

        # Update the click manifest (we still support click format)
        if value is None:
            if app in self.test_bin_path:
                self.test_bin_path.pop(app)
        else:
            if app not in self.test_bin_path:
                self.test_bin_path[app] = dict()
            self.test_bin_path[app] = value

        # Now update TEST_BIN_PATH
        self._update_test_bin_path()

    def set_test_framework(self, app, key, value):
        '''Set framework entries. If value is None, remove key, if key is
           None, remove framework from manifest'''
        if key is None:
            if app in self.test_framework:
                self.test_framework.pop(app)
        elif value is None:
            if key in self.test_framework[app]:
                self.test_framework[app].pop(key)
        else:
            if app not in self.test_framework:
                self.test_framework[app] = dict()
            self.test_framework[app][key] = value
        self._update_test_framework()

    def set_test_framework_policy(self, policy_dict=None):
        '''Set framework policy'''
        if policy_dict is None:  # Reset
            self.test_framework_policy = dict()
            for i in ['apparmor', 'seccomp']:
                self.test_framework_policy[i] = dict()
                for j in ['templates', 'policygroups']:
                    self.test_framework_policy[i][j] = dict()
                    for k in ['-common', '-reserved']:
                        n = "%s%s" % (j.rstrip('s'), k)
                        self.test_framework_policy[i][j][n] = \
                            '''# Description: %s
# Usage: %s
''' % (n, k.lstrip('-'))
        else:
            self.test_framework_policy = policy_dict
        self._update_test_framework_policy()

    def set_test_framework_policy_unknown(self, unknown=[]):
        '''Set framework policy unknown'''
        self.test_framework_policy_unknown = unknown
        self._update_test_framework_policy_unknown()

    def set_test_systemd(self, app, key, value):
        '''Set systemd entries. If key is None, remove snappy-systemd from
           yaml.

           pkg_yaml['services'][*]['name'] = app
           pkg_yaml['services'][*][key] = value
        '''

        # Update the package.yaml
        if key is None:
            if 'services' in self.test_pkg_yaml:
                for s in self.test_pkg_yaml['services']:
                    if 'name' in s and s['name'] == app:
                        self.test_pkg_yaml['services'].remove(s)
                        break
        else:
            found = False
            if 'services' in self.test_pkg_yaml:
                for s in self.test_pkg_yaml['services']:
                    if 'name' in s and s['name'] == app:
                        # Found the entry, so update key/value
                        s[key] = value
                        found = True
                        break
            # Did not find the entry, so create one
            if not found:
                if 'services' not in self.test_pkg_yaml:
                    self.test_pkg_yaml['services'] = []
                self.test_pkg_yaml['services'].append({'name': app,
                                                       key: value})
        self._update_test_pkg_yaml()

    def setUp(self):
        '''Make sure our patches are applied everywhere'''
        patches = create_patches()
        for p in patches:
            p.start()
            self.addCleanup(p.stop)

    def tearDown(self):
        '''Make sure we reset everything to known good values'''
        global TEST_CONTROL
        TEST_CONTROL = ""
        global TEST_MANIFEST
        TEST_MANIFEST = ""
        global TEST_PKG_YAML
        TEST_PKG_YAML = ""
        global TEST_HASHES_YAML
        TEST_HASHES_YAML = ""
        global TEST_README_MD
        TEST_README_MD = ""
        global TEST_SECURITY
        TEST_SECURITY = dict()
        global TEST_SECURITY_PROFILES
        TEST_SECURITY_PROFILES = dict()
        global TEST_DESKTOP
        TEST_DESKTOP = dict()
        global TEST_URLS
        TEST_URLS = dict()
        global TEST_SCOPES
        TEST_SCOPES = dict()
        global TEST_CONTENT_HUB
        TEST_CONTENT_HUB = dict()
        global TEST_ACCOUNTS_MANIFEST
        TEST_ACCOUNTS_MANIFEST = dict()
        global TEST_ACCOUNTS_APPLICATION
        TEST_ACCOUNTS_APPLICATION = dict()
        global TEST_ACCOUNTS_PROVIDER
        TEST_ACCOUNTS_PROVIDER = dict()
        global TEST_ACCOUNTS_QML_PLUGIN
        TEST_ACCOUNTS_QML_PLUGIN = dict()
        global TEST_ACCOUNTS_SERVICE
        TEST_ACCOUNTS_SERVICE = dict()
        global TEST_PUSH_HELPER
        TEST_PUSH_HELPER = dict()
        global TEST_BIN_PATH
        TEST_BIN_PATH = dict()
        global TEST_FRAMEWORK
        TEST_FRAMEWORK = dict()
        global TEST_FRAMEWORK_POLICY
        TEST_FRAMEWORK_POLICY = dict()
        global TEST_FRAMEWORK_POLICY_UNKNOWN
        TEST_FRAMEWORK_POLICY_UNKNOWN = []
        global TEST_PKGFMT_TYPE
        TEST_PKGFMT_TYPE = "click"
        global TEST_PKGFMT_VERSION
        TEST_PKGFMT_VERSION = "0.4"

        self._reset_test_data()
        common.recursive_rm(self.desktop_tmpdir)



import time
from unittest import TestCase
from unittest.mock import patch

from clickreviews.remote import UPDATE_INTERVAL, _update_is_necessary


class RemoteTestCase(TestCase):
    def patch_path(self):
        p = patch('clickreviews.remote.os.path')
        self.mock_path = p.start()
        self.addCleanup(p.stop)

    def patch_time(self, now):
        p = patch('clickreviews.remote.time.time')
        self.mock_time = p.start()
        self.mock_time.return_value = now
        self.addCleanup(p.stop)

    def test_no_update_needed(self):
        now = time.time()
        self.patch_time(now)
        self.patch_path()

        # last update was 10 seconds ago
        self.mock_path.getmtime.return_value = now - 10

        self.assertFalse(_update_is_necessary('some-file'))

    def test_update_needed(self):
        now = time.time()
        self.patch_time(now)
        self.patch_path()

        # last update was UPDATE_INTERVAL + 10 seconds ago
        self.mock_path.getmtime.return_value = now - UPDATE_INTERVAL - 10

        self.assertTrue(_update_is_necessary('some-file'))



'''cr_language_packs.py: click language packs'''
#
# Copyright (C) 2018 The UBports Foundation
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function

from clickreviews.cr_common import ClickReview, error, open_file_read
import json
import os


class ClickReviewLanguagePacks(ClickReview):
    '''This class represents click lint reviews'''
    def __init__(self, fn, overrides=None):
        peer_hooks = dict()
        my_hook = 'language-packs'
        peer_hooks[my_hook] = dict()
        peer_hooks[my_hook]['allowed'] = ['apparmor']
        peer_hooks[my_hook]['required'] = []

        ClickReview.__init__(self,
                             fn,
                             "language_packs",
                             peer_hooks=peer_hooks,
                             overrides=overrides)
        if not self.is_click and not self.is_snap1:
            return

        self.language_packs_files = dict()
        self.language_packs = dict()

        if self.manifest is None:
            return

        for app in self.manifest['hooks']:
            if my_hook not in self.manifest['hooks'][app]:
                # msg("Skipped missing %s hook for '%s'" % (my_hook, app))
                continue
            if not isinstance(self.manifest['hooks'][app][my_hook], str):
                error("manifest malformed: hooks/%s/%s is not a str" % (
                      app, my_hook))

            (full_fn, parsed) = self._extract_language_pack(app)
            self.language_packs_files[app] = full_fn
            self.language_packs[app] = parsed

    def _extract_language_pack(self, app):
        '''Extract language pack'''
        if self.manifest is None:
            return

        a = self.manifest['hooks'][app]['language-packs']
        fn = os.path.join(self.unpack_dir, a)

        bn = os.path.basename(fn)
        if not os.path.exists(fn):
            error("Could not find '%s'" % bn)

        fh = open_file_read(fn)
        contents = ""
        for line in fh.readlines():
            contents += line
        fh.close()

        try:
            jd = json.loads(contents)
        except Exception as e:
            error("language pack json unparseable: %s (%s):\n%s" % (bn,
                  str(e), contents))

        if not isinstance(jd, dict):
            error("language pack json is malformed: %s:\n%s" % (bn, contents))

        return (fn, jd)

    def check_manifest(self):
        '''Check manifest'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.language_packs.keys()):
            t = 'error'
            n = self._get_check_name('manifest', app=app)
            s = "(NEEDS REVIEW) Language packs need manual review"
            # Return always valid, but request manual review. This is in order
            # to avoid having multiple language packs for the same language,
            # uploaded by different users.
            manual_review = True
            self._add_result(t, n, s, manual_review=manual_review)
